# SPDX-FileCopyrightText: 2025 QinHan
# SPDX-License-Identifier: MPL-2.0
"""
Allow running docutranslate mcp server with `python -m docutranslate.mcp`
"""

from docutranslate.mcp.server import main

if __name__ == "__main__":
    main()



# SPDX-FileCopyrightText: 2025 QinHan
# SPDX-License-Identifier: MPL-2.0
"""
DocuTranslate Shared Server Layer

This module provides the shared TranslationService that encapsulates all core
translation task logic, used by both the Web backend (app.py) and MCP server.
"""

import asyncio
import base64
import json
import logging
import os
import shutil
import tempfile
import time
import uuid
from pathlib import Path
from typing import (
    List,
    Dict,
    Any,
    Optional,
    Type,
)

import httpx
from fastapi import HTTPException
from pydantic import TypeAdapter

from docutranslate import __version__
from docutranslate.agents.glossary_agent import GlossaryAgentConfig
from docutranslate.core.schemas import TranslatePayload
from docutranslate.exporter.md.types import ConvertEngineType
from docutranslate.global_values.conditional_import import DOCLING_EXIST
from docutranslate.workflow.ass_workflow import AssWorkflow, AssWorkflowConfig
from docutranslate.workflow.base import Workflow
from docutranslate.workflow.docx_workflow import DocxWorkflow, DocxWorkflowConfig
from docutranslate.workflow.epub_workflow import EpubWorkflow, EpubWorkflowConfig
from docutranslate.workflow.html_workflow import HtmlWorkflow, HtmlWorkflowConfig
from docutranslate.workflow.interfaces import (
    DocxExportable,
    EpubExportable,
    HTMLExportable,
    MDFormatsExportable,
    TXTExportable,
    JsonExportable,
    XlsxExportable,
    SrtExportable,
    CsvExportable,
    AssExportable,
    PPTXExportable,
)
from docutranslate.workflow.json_workflow import JsonWorkflow, JsonWorkflowConfig
from docutranslate.workflow.md_based_workflow import (
    MarkdownBasedWorkflow,
    MarkdownBasedWorkflowConfig,
)
from docutranslate.workflow.pptx_workflow import PPTXWorkflow, PPTXWorkflowConfig
from docutranslate.workflow.srt_workflow import SrtWorkflow, SrtWorkflowConfig
from docutranslate.workflow.txt_workflow import TXTWorkflow, TXTWorkflowConfig
from docutranslate.workflow.xlsx_workflow import XlsxWorkflow, XlsxWorkflowConfig
from docutranslate.logger import global_logger
from docutranslate.progress import ProgressTracker
from docutranslate.translator import default_params

if DOCLING_EXIST:
    from docutranslate.converter.x2md.converter_docling import ConverterDoclingConfig
from docutranslate.converter.x2md.converter_mineru import ConverterMineruConfig
from docutranslate.converter.x2md.converter_mineru_deploy import ConverterMineruDeployConfig
from docutranslate.exporter.md.md2html_exporter import MD2HTMLExporterConfig
from docutranslate.exporter.md.md2docx_exporter import MD2DocxExporterConfig
from docutranslate.exporter.txt.txt2html_exporter import TXT2HTMLExporterConfig
from docutranslate.translator.ai_translator.md_translator import MDTranslatorConfig
from docutranslate.translator.ai_translator.txt_translator import TXTTranslatorConfig
from docutranslate.translator.ai_translator.json_translator import JsonTranslatorConfig
from docutranslate.exporter.js.json2html_exporter import Json2HTMLExporterConfig
from docutranslate.translator.ai_translator.xlsx_translator import XlsxTranslatorConfig
from docutranslate.exporter.xlsx.xlsx2html_exporter import Xlsx2HTMLExporterConfig
from docutranslate.translator.ai_translator.docx_translator import DocxTranslatorConfig
from docutranslate.exporter.docx.docx2html_exporter import Docx2HTMLExporterConfig
from docutranslate.translator.ai_translator.srt_translator import SrtTranslatorConfig
from docutranslate.exporter.srt.srt2html_exporter import Srt2HTMLExporterConfig
from docutranslate.translator.ai_translator.epub_translator import EpubTranslatorConfig
from docutranslate.exporter.epub.epub2html_exporter import Epub2HTMLExporterConfig
from docutranslate.translator.ai_translator.html_translator import HtmlTranslatorConfig
from docutranslate.translator.ai_translator.ass_translator import AssTranslatorConfig
from docutranslate.exporter.ass.ass2html_exporter import Ass2HTMLExporterConfig
from docutranslate.translator.ai_translator.pptx_translator import PPTXTranslatorConfig
from docutranslate.exporter.pptx.pptx2html_exporter import PPTX2HTMLExporterConfig


MAX_LOG_HISTORY = 200


# --- Workflow dictionary ---
WORKFLOW_DICT: Dict[str, Type[Workflow]] = {
    "markdown_based": MarkdownBasedWorkflow,
    "txt": TXTWorkflow,
    "json": JsonWorkflow,
    "xlsx": XlsxWorkflow,
    "docx": DocxWorkflow,
    "srt": SrtWorkflow,
    "epub": EpubWorkflow,
    "html": HtmlWorkflow,
    "ass": AssWorkflow,
    "pptx": PPTXWorkflow,
}


# --- Media types mapping ---
MEDIA_TYPES = {
    "html": "text/html; charset=utf-8",
    "markdown": "text/markdown; charset=utf-8",
    "markdown_zip": "application/zip",
    "txt": "text/plain; charset=utf-8",
    "json": "application/json; charset=utf-8",
    "xlsx": "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet",
    "csv": "text/csv; charset=utf-8",
    "docx": "application/vnd.openxmlformats-officedocument.wordprocessingml.document",
    "srt": "text/plain; charset=utf-8",
    "epub": "application/epub+zip",
    "ass": "text/plain; charset=utf-8",
    "pptx": "application/vnd.openxmlformats-officedocument.presentationml.presentation",
}


class QueueAndHistoryHandler(logging.Handler):
    """Logging handler that stores logs in both a history list and an asyncio.Queue."""

    def __init__(
        self,
        queue_ref: asyncio.Queue,
        history_list_ref: List[str],
        max_history_items: int,
        task_id: str,
    ):
        super().__init__()
        self.queue = queue_ref
        self.history_list = history_list_ref
        self.max_history = max_history_items
        self.task_id = task_id

    def emit(self, record: logging.LogRecord):
        log_entry = self.format(record)
        print(f"[{self.task_id}] {log_entry}")
        self.history_list.append(log_entry)
        if len(self.history_list) > self.max_history:
            del self.history_list[: len(self.history_list) - self.max_history]
        if self.queue is not None:
            try:
                # Try to get the main event loop - this will be set by the application
                self.queue.put_nowait(log_entry)
            except asyncio.QueueFull:
                print(f"[{self.task_id}] Log queue is full. Log dropped: {log_entry}")
            except Exception as e:
                print(
                    f"[{self.task_id}] Error putting log to queue: {e}. Log: {log_entry}"
                )


def get_workflow_type_from_filename(filename: str) -> str:
    """Get workflow type based on file extension."""
    ext = Path(filename).suffix.lower()
    if ext in [".pdf", ".png", ".jpg"]:
        return "markdown_based"
    elif ext in [".md", ".markdown"]:
        return "markdown_based"
    elif ext in [".docx", ".doc"]:
        return "docx"
    elif ext in [".csv", ".xlsx", ".xls"]:
        return "xlsx"
    elif ext in [".pptx", ".ppt"]:
        return "pptx"
    elif ext in [".json"]:
        return "json"
    elif ext in [".srt"]:
        return "srt"
    elif ext in [".ass"]:
        return "ass"
    elif ext in [".epub"]:
        return "epub"
    elif ext in [".html", ".htm"]:
        return "html"
    elif ext in [".txt"]:
        return "txt"
    else:
        return "txt"


def _create_default_task_state() -> Dict[str, Any]:
    """Create a new default task state."""
    return {
        "is_processing": False,
        "status_message": "空闲",
        "error_flag": False,
        "download_ready": False,
        "progress_percent": 0,
        "workflow_instance": None,
        "original_filename_stem": None,
        "task_start_time": 0,
        "task_end_time": 0,
        "current_task_ref": None,
        "original_filename": None,
        "temp_dir": None,
        "downloadable_files": {},
        "attachment_files": {},
    }


class TranslationService:
    """
    Shared translation service that provides core task management functionality.

    This class is used by both the Web backend (app.py) and MCP server to ensure
    consistent task management across both interfaces.
    """

    def __init__(self):
        # Task state storage
        self.tasks_state: Dict[str, Dict[str, Any]] = {}
        self.tasks_log_queues: Dict[str, asyncio.Queue] = {}
        self.tasks_log_histories: Dict[str, List[str]] = {}

        # HTTP client for CDN checks
        self.httpx_client: Optional[httpx.AsyncClient] = None

        # Lock for thread-safe operations
        self._lock = asyncio.Lock()

        # Reference to main event loop (set by application)
        self.main_event_loop: Optional[asyncio.AbstractEventLoop] = None

    def initialize(self, httpx_client: httpx.AsyncClient, main_event_loop: asyncio.AbstractEventLoop):
        """Initialize the service with HTTP client and event loop."""
        self.httpx_client = httpx_client
        self.main_event_loop = main_event_loop

    def clear_all(self):
        """Clear all task states (called during application startup)."""
        self.tasks_state.clear()
        self.tasks_log_queues.clear()
        self.tasks_log_histories.clear()

    def list_tasks(self) -> List[str]:
        """List all task IDs."""
        return list(self.tasks_state.keys())

    def get_task_state(self, task_id: str) -> Optional[Dict[str, Any]]:
        """Get task state by ID."""
        return self.tasks_state.get(task_id)

    def get_task_logs(self, task_id: str) -> List[str]:
        """Get task log history by ID."""
        return self.tasks_log_histories.get(task_id, [])

    async def get_new_logs(self, task_id: str) -> List[str]:
        """Get new logs from the queue."""
        if task_id not in self.tasks_log_queues:
            raise HTTPException(
                status_code=404, detail=f"找不到任务ID '{task_id}' 的日志队列。"
            )
        log_queue = self.tasks_log_queues[task_id]
        new_logs = []
        while not log_queue.empty():
            try:
                new_logs.append(log_queue.get_nowait())
                log_queue.task_done()
            except asyncio.QueueEmpty:
                break
        return new_logs

    def get_downloadable_file_path(self, task_id: str, file_type: str) -> Optional[Dict[str, str]]:
        """Get downloadable file info."""
        task_state = self.tasks_state.get(task_id)
        if not task_state:
            return None
        return task_state.get("downloadable_files", {}).get(file_type)

    def get_attachment_file_path(self, task_id: str, identifier: str) -> Optional[Dict[str, str]]:
        """Get attachment file info."""
        task_state = self.tasks_state.get(task_id)
        if not task_state:
            return None
        return task_state.get("attachment_files", {}).get(identifier)

    async def start_translation(
        self,
        task_id: str,
        payload: TranslatePayload,
        file_contents: bytes,
        original_filename: str,
    ) -> Dict[str, Any]:
        """
        Start a translation task.

        Args:
            task_id: Unique task identifier
            payload: Translation parameters
            file_contents: File content bytes
            original_filename: Original filename

        Returns:
            Response dict with task_id and status
        """
        # Auto workflow routing
        if payload.workflow_type == "auto":
            detected_type = get_workflow_type_from_filename(original_filename)
            print(f"[{task_id}] 自动识别工作流: {original_filename} -> {detected_type}")

            # 关键修复：完全手动构造 payload_data，不依赖 model_dump
            # 这样可以确保所有字段都正确传递，不会因为 exclude_none 或其他原因丢失
            payload_data = {
                "workflow_type": detected_type,
            }

            # 从 BaseWorkflowParams 复制所有字段
            base_fields = [
                "skip_translate", "base_url", "api_key", "model_id", "to_lang",
                "chunk_size", "concurrent", "temperature", "timeout", "thinking", "retry",
                "system_proxy_enable", "custom_prompt", "glossary_dict",
                "glossary_generate_enable", "glossary_agent_config",
                "force_json", "rpm", "tpm", "provider", "extra_body"
            ]
            for field_name in base_fields:
                if hasattr(payload, field_name):
                    value = getattr(payload, field_name)
                    if value is not None and value != "":
                        payload_data[field_name] = value

            # 从 UniversalParamsMixin 复制所有字段（关键！）
            universal_fields = [
                "convert_engine", "mineru_token", "model_version", "formula_ocr", "code_ocr",
                "mineru_deploy_base_url", "mineru_deploy_backend", "mineru_deploy_parse_method",
                "mineru_deploy_table_enable", "mineru_deploy_formula_enable",
                "mineru_deploy_start_page_id", "mineru_deploy_end_page_id",
                "mineru_deploy_lang_list", "mineru_deploy_server_url",
                "insert_mode", "separator", "translate_regions", "json_paths", "md2docx_engine"
            ]
            for field_name in universal_fields:
                if hasattr(payload, field_name):
                    value = getattr(payload, field_name)
                    if value is not None:
                        # 注意：mineru_token 即使是空字符串也要保留，因为 MarkdownWorkflowParams 的默认值是空字符串
                        # 但如果用户明确传入了 token（非空），我们需要保留它
                        if isinstance(value, str) and value == "" and field_name != "mineru_token":
                            continue  # 跳过空字符串，除了 mineru_token
                        payload_data[field_name] = value
                        print(f"[{task_id}] 复制字段 {field_name}: {type(value).__name__}" +
                              (f" (len={len(value)})" if isinstance(value, str) else ""))

            # 调试日志
            print(f"[{task_id}] payload_data keys: {sorted(payload_data.keys())}")
            if "mineru_token" in payload_data:
                token = payload_data["mineru_token"]
                print(f"[{task_id}] mineru_token in payload_data (length: {len(token)}, starts with: {token[:20] if len(token) > 20 else token}...)")
            if "convert_engine" in payload_data:
                print(f"[{task_id}] convert_engine: {payload_data['convert_engine']}")

            if detected_type == "json" and not payload_data.get("json_paths"):
                payload_data["json_paths"] = ["$..*"]

            if detected_type == "markdown_based" and not payload_data.get("convert_engine"):
                if Path(original_filename).suffix.lower() == ".pdf":
                    payload_data["convert_engine"] = "mineru" if not DOCLING_EXIST else "docling"
                else:
                    payload_data["convert_engine"] = "identity"

            try:
                payload = TypeAdapter(TranslatePayload).validate_python(payload_data)
                # 验证后再次检查
                if hasattr(payload, "mineru_token"):
                    token = payload.mineru_token
                    print(f"[{task_id}] After validation: mineru_token present (length: {len(token) if token else 0})")
                    if token:
                        print(f"[{task_id}] mineru_token starts with: {token[:20] if len(token) > 20 else token}")
                if hasattr(payload, "convert_engine"):
                    print(f"[{task_id}] After validation: convert_engine={payload.convert_engine}")
            except Exception as e:
                raise HTTPException(status_code=400, detail=f"自动转换工作流参数失败: {e}")

        if task_id not in self.tasks_state:
            self.tasks_state[task_id] = _create_default_task_state()
            self.tasks_log_queues[task_id] = asyncio.Queue()
            self.tasks_log_histories[task_id] = []
        task_state = self.tasks_state[task_id]

        if (
            task_state["is_processing"]
            and task_state["current_task_ref"]
            and not task_state["current_task_ref"].done()
        ):
            raise HTTPException(
                status_code=429, detail=f"任务ID '{task_id}' 正在进行中，请稍后再试。"
            )

        if task_state.get("temp_dir") and os.path.isdir(task_state["temp_dir"]):
            shutil.rmtree(task_state["temp_dir"])

        raw_stem = Path(original_filename).stem
        safe_stem = raw_stem[:50] if len(raw_stem) > 50 else raw_stem

        task_state.update(
            {
                "is_processing": True,
                "status_message": "任务初始化中...",
                "error_flag": False,
                "download_ready": False,
                "workflow_instance": None,
                "original_filename_stem": safe_stem,
                "original_filename": original_filename,
                "task_start_time": time.time(),
                "task_end_time": 0,
                "current_task_ref": None,
                "temp_dir": None,
                "downloadable_files": {},
                "attachment_files": {},
            }
        )

        log_history = self.tasks_log_histories[task_id]
        log_queue = self.tasks_log_queues[task_id]
        log_history.clear()
        while not log_queue.empty():
            try:
                log_queue.get_nowait()
            except asyncio.QueueEmpty:
                break

        initial_log_msg = f"收到新的翻译请求: {original_filename}"
        print(f"[{task_id}] {initial_log_msg}")
        await log_queue.put(initial_log_msg)

        try:
            loop = asyncio.get_running_loop()
            task = loop.create_task(
                self._perform_translation(task_id, payload, file_contents, original_filename)
            )
            task_state["current_task_ref"] = task
            return {
                "task_started": True,
                "task_id": task_id,
                "message": "翻译任务已成功启动，请稍候...",
            }
        except Exception as e:
            task_state.update(
                {
                    "is_processing": False,
                    "status_message": f"启动任务失败: {e}",
                    "error_flag": True,
                    "current_task_ref": None,
                }
            )
            raise HTTPException(status_code=500, detail=f"启动翻译任务时出错: {e}")

    async def _perform_translation(
        self,
        task_id: str,
        payload: TranslatePayload,
        file_contents: bytes,
        original_filename: str,
    ):
        """Perform the actual translation work."""
        task_state = self.tasks_state[task_id]
        log_queue = self.tasks_log_queues[task_id]
        log_history = self.tasks_log_histories[task_id]

        task_logger = logging.getLogger(f"task.{task_id}")
        task_logger.setLevel(logging.INFO)
        task_logger.propagate = False
        if task_logger.hasHandlers():
            task_logger.handlers.clear()
        task_handler = QueueAndHistoryHandler(
            log_queue, log_history, MAX_LOG_HISTORY, task_id=task_id
        )
        task_handler.setFormatter(
            logging.Formatter("%(asctime)s - %(levelname)s - %(message)s")
        )
        task_logger.addHandler(task_handler)

        task_logger.info(
            f"后台翻译任务开始: 文件 '{original_filename}', 工作流: '{payload.workflow_type}'"
        )
        task_state["status_message"] = f"正在处理 '{original_filename}'..."

        def update_progress(percent: int, message: str):
            task_state["progress_percent"] = percent
            if message:
                task_state["status_message"] = message

        progress_tracker = ProgressTracker(
            logger=task_logger,
            callback=update_progress
        )

        temp_dir = None

        try:
            workflow_class = WORKFLOW_DICT.get(payload.workflow_type)
            if not workflow_class:
                raise ValueError(f"不支持的工作流类型: '{payload.workflow_type}'")

            workflow: Workflow

            def build_glossary_agent_config():
                if payload.glossary_generate_enable and payload.glossary_agent_config:
                    agent_payload = payload.glossary_agent_config
                    return GlossaryAgentConfig(
                        logger=task_logger, **agent_payload.model_dump()
                    )
                return None

            if hasattr(payload, 'md2docx_engine'):
                md2docx_engine = payload.md2docx_engine
            else:
                md2docx_engine = "auto"

            workflow = self._create_workflow(
                payload, task_logger, progress_tracker, build_glossary_agent_config, md2docx_engine
            )

            file_stem = task_state["original_filename_stem"]
            file_suffix = Path(original_filename).suffix
            workflow.read_bytes(content=file_contents, stem=file_stem, suffix=file_suffix)
            await workflow.translate_async()

            task_logger.info("翻译完成，正在生成临时结果文件...")
            temp_dir = tempfile.mkdtemp(prefix=f"docutranslate_{task_id}_")
            task_state["temp_dir"] = temp_dir
            downloadable_files = {}
            filename_stem = task_state["original_filename_stem"]

            is_cdn_available = True
            try:
                await self.httpx_client.head(
                    "https://s4.zstatic.net/ajax/libs/KaTeX/0.16.9/contrib/auto-render.min.js",
                    timeout=3,
                )
            except (httpx.TimeoutException, httpx.RequestError):
                is_cdn_available = False
                task_logger.warning("CDN连接失败，将使用本地JS进行渲染。")

            export_map = self._build_export_map(
                workflow, filename_stem, is_cdn_available
            )

            for file_type, (export_func, filename, is_string_output) in export_map.items():
                try:
                    content = await asyncio.to_thread(export_func)
                    content_bytes = content.encode("utf-8") if is_string_output else content
                    file_path = os.path.join(temp_dir, filename)
                    with open(file_path, "wb") as f:
                        f.write(content_bytes)
                    downloadable_files[file_type] = {
                        "path": file_path,
                        "filename": filename,
                    }
                    task_logger.info(f"成功生成 {file_type} 文件")
                except Exception as export_error:
                    task_logger.error(
                        f"生成 {file_type} 文件时出错: {export_error}", exc_info=True
                    )

            attachment_files = {}
            attachment_object = workflow.get_attachment()
            if attachment_object and attachment_object.attachment_dict:
                task_logger.info(
                    f"发现 {len(attachment_object.attachment_dict)} 个附件，正在处理..."
                )
                for identifier, doc in attachment_object.attachment_dict.items():
                    try:
                        attachment_filename = f"{doc.stem or identifier}{doc.suffix}"
                        attachment_path = os.path.join(temp_dir, attachment_filename)
                        with open(attachment_path, "wb") as f:
                            f.write(doc.content)
                        attachment_files[identifier] = {
                            "path": attachment_path,
                            "filename": attachment_filename,
                        }
                        task_logger.info(
                            f"成功生成附件 '{identifier}' 文件: {attachment_filename}"
                        )
                    except Exception as attachment_error:
                        task_logger.error(
                            f"生成附件 '{identifier}' 文件时出错: {attachment_error}",
                            exc_info=True,
                        )

            end_time = time.time()
            duration = end_time - task_state["task_start_time"]
            task_state.update(
                {
                    "status_message": f"翻译成功！用时 {duration:.2f} 秒。",
                    "download_ready": True,
                    "error_flag": False,
                    "progress_percent": 100,
                    "task_end_time": end_time,
                    "downloadable_files": downloadable_files,
                    "attachment_files": attachment_files,
                }
            )
            task_logger.info(f"翻译成功完成，用时 {duration:.2f} 秒。")

        except asyncio.CancelledError:
            end_time = time.time()
            duration = end_time - task_state["task_start_time"]
            task_logger.info(
                f"翻译任务 '{original_filename}' 已被取消 (用时 {duration:.2f} 秒)."
            )
            task_state.update(
                {
                    "status_message": f"翻译任务已取消 (用时 {duration:.2f} 秒).",
                    "error_flag": False,
                    "download_ready": False,
                    "progress_percent": 100,
                    "task_end_time": end_time,
                }
            )
        except Exception as e:
            end_time = time.time()
            duration = end_time - task_state["task_start_time"]
            error_message = f"翻译失败: {e}"
            task_logger.error(error_message, exc_info=True)
            task_state.update(
                {
                    "status_message": f"翻译过程中发生错误 (用时 {duration:.2f} 秒): {e}",
                    "error_flag": True,
                    "download_ready": False,
                    "progress_percent": 100,
                    "task_end_time": end_time,
                }
            )
        finally:
            task_state["workflow_instance"] = None
            task_state["is_processing"] = False
            task_state["current_task_ref"] = None

            if task_state["error_flag"] and temp_dir and os.path.isdir(temp_dir):
                shutil.rmtree(temp_dir)
                task_logger.info(f"因任务失败，已清理临时目录")
                task_state["temp_dir"] = None

            task_logger.info(f"后台翻译任务 '{original_filename}' 处理结束。")
            task_logger.removeHandler(task_handler)

    def _create_workflow(
        self,
        payload: TranslatePayload,
        task_logger: logging.Logger,
        progress_tracker: ProgressTracker,
        build_glossary_agent_config,
        md2docx_engine: str,
    ) -> Workflow:
        """Create workflow instance based on payload type."""
        from docutranslate.core.schemas import (
            MarkdownWorkflowParams,
            TextWorkflowParams,
            JsonWorkflowParams,
            XlsxWorkflowParams,
            DocxWorkflowParams,
            SrtWorkflowParams,
            EpubWorkflowParams,
            HtmlWorkflowParams,
            AssWorkflowParams,
            PPTXWorkflowParams,
        )

        if isinstance(payload, MarkdownWorkflowParams):
            task_logger.info("构建 MarkdownBasedWorkflow 配置。")
            translator_args = payload.model_dump(
                include={
                    "skip_translate",
                    "base_url",
                    "api_key",
                    "model_id",
                    "to_lang",
                    "custom_prompt",
                    "temperature",
                    "thinking",
                    "chunk_size",
                    "concurrent",
                    "glossary_dict",
                    "timeout",
                    "retry",
                    "system_proxy_enable",
                    "force_json",
                    "rpm",
                    "tpm",
                    "provider",
                    "extra_body",
                },
                exclude_none=True,
            )
            translator_args["glossary_generate_enable"] = payload.glossary_generate_enable
            translator_args["glossary_agent_config"] = build_glossary_agent_config()
            translator_config = MDTranslatorConfig(**translator_args)
            translator_config.progress_tracker = progress_tracker

            converter_config = None
            if payload.convert_engine == "mineru":
                token = payload.mineru_token or ""
                task_logger.info(f"Creating ConverterMineruConfig with mineru_token (length: {len(token)})")
                if token:
                    task_logger.info(f"mineru_token starts with: {token[:20] if len(token) > 20 else token}")
                converter_config = ConverterMineruConfig(
                    logger=task_logger,
                    mineru_token=token,
                    formula_ocr=payload.formula_ocr,
                    model_version=payload.model_version,
                )
            elif payload.convert_engine == "mineru_deploy":
                converter_config = ConverterMineruDeployConfig(
                    base_url=payload.mineru_deploy_base_url,
                    backend=payload.mineru_deploy_backend,
                    parse_method=payload.mineru_deploy_parse_method,
                    formula_enable=payload.mineru_deploy_formula_enable,
                    table_enable=payload.mineru_deploy_table_enable,
                    start_page_id=payload.mineru_deploy_start_page_id,
                    end_page_id=payload.mineru_deploy_end_page_id,
                    lang_list=payload.mineru_deploy_lang_list,
                    server_url=payload.mineru_deploy_server_url,
                )
            elif payload.convert_engine == "docling" and DOCLING_EXIST:
                converter_config = ConverterDoclingConfig(
                    logger=task_logger,
                    code_ocr=payload.code_ocr,
                    formula_ocr=payload.formula_ocr,
                )
            html_exporter_config = MD2HTMLExporterConfig(cdn=True)
            md2docx_exporter_config = MD2DocxExporterConfig(
                engine=md2docx_engine
            ) if md2docx_engine is not None else None
            workflow_config = MarkdownBasedWorkflowConfig(
                convert_engine=payload.convert_engine,
                converter_config=converter_config,
                translator_config=translator_config,
                html_exporter_config=html_exporter_config,
                md2docx_exporter_config=md2docx_exporter_config,
                logger=task_logger,
                progress_tracker=progress_tracker,
            )
            return MarkdownBasedWorkflow(config=workflow_config)

        elif isinstance(payload, TextWorkflowParams):
            task_logger.info("构建 TXTWorkflow 配置。")
            translator_args = payload.model_dump(
                include={
                    "skip_translate",
                    "base_url",
                    "api_key",
                    "model_id",
                    "to_lang",
                    "custom_prompt",
                    "temperature",
                    "thinking",
                    "chunk_size",
                    "concurrent",
                    "glossary_dict",
                    "insert_mode",
                    "separator",
                    "segment_mode",
                    "timeout",
                    "retry",
                    "system_proxy_enable",
                    "force_json",
                    "rpm",
                    "tpm",
                    "provider",
                    "extra_body",
                },
                exclude_none=True,
            )
            translator_args["glossary_generate_enable"] = payload.glossary_generate_enable
            translator_args["glossary_agent_config"] = build_glossary_agent_config()
            translator_config = TXTTranslatorConfig(**translator_args)
            translator_config.progress_tracker = progress_tracker

            html_exporter_config = TXT2HTMLExporterConfig(cdn=True)
            workflow_config = TXTWorkflowConfig(
                translator_config=translator_config,
                html_exporter_config=html_exporter_config,
                logger=task_logger,
                progress_tracker=progress_tracker,
            )
            return TXTWorkflow(config=workflow_config)

        elif isinstance(payload, JsonWorkflowParams):
            task_logger.info("构建 JsonWorkflow 配置。")
            translator_args = payload.model_dump(
                include={
                    "skip_translate",
                    "base_url",
                    "api_key",
                    "model_id",
                    "to_lang",
                    "custom_prompt",
                    "temperature",
                    "thinking",
                    "chunk_size",
                    "concurrent",
                    "glossary_dict",
                    "json_paths",
                    "timeout",
                    "retry",
                    "system_proxy_enable",
                    "force_json",
                    "rpm",
                    "tpm",
                    "provider",
                    "extra_body",
                },
                exclude_none=True,
            )
            translator_args["glossary_generate_enable"] = payload.glossary_generate_enable
            translator_args["glossary_agent_config"] = build_glossary_agent_config()
            translator_config = JsonTranslatorConfig(**translator_args)
            translator_config.progress_tracker = progress_tracker

            html_exporter_config = Json2HTMLExporterConfig(cdn=True)
            workflow_config = JsonWorkflowConfig(
                translator_config=translator_config,
                html_exporter_config=html_exporter_config,
                logger=task_logger,
                progress_tracker=progress_tracker,
            )
            return JsonWorkflow(config=workflow_config)

        elif isinstance(payload, XlsxWorkflowParams):
            task_logger.info("构建 XlsxWorkflow 配置。")
            translator_args = payload.model_dump(
                include={
                    "skip_translate",
                    "base_url",
                    "api_key",
                    "model_id",
                    "to_lang",
                    "custom_prompt",
                    "temperature",
                    "thinking",
                    "chunk_size",
                    "concurrent",
                    "insert_mode",
                    "separator",
                    "translate_regions",
                    "glossary_dict",
                    "timeout",
                    "retry",
                    "system_proxy_enable",
                    "force_json",
                    "rpm",
                    "tpm",
                    "provider",
                    "extra_body",
                },
                exclude_none=True,
            )
            translator_args["glossary_generate_enable"] = payload.glossary_generate_enable
            translator_args["glossary_agent_config"] = build_glossary_agent_config()
            translator_config = XlsxTranslatorConfig(**translator_args)
            translator_config.progress_tracker = progress_tracker

            html_exporter_config = Xlsx2HTMLExporterConfig(cdn=True)
            workflow_config = XlsxWorkflowConfig(
                translator_config=translator_config,
                html_exporter_config=html_exporter_config,
                logger=task_logger,
                progress_tracker=progress_tracker,
            )
            return XlsxWorkflow(config=workflow_config)

        elif isinstance(payload, DocxWorkflowParams):
            task_logger.info("构建 DocxWorkflow 配置。")
            translator_args = payload.model_dump(
                include={
                    "skip_translate",
                    "base_url",
                    "api_key",
                    "model_id",
                    "to_lang",
                    "custom_prompt",
                    "temperature",
                    "thinking",
                    "chunk_size",
                    "concurrent",
                    "insert_mode",
                    "separator",
                    "glossary_dict",
                    "timeout",
                    "retry",
                    "system_proxy_enable",
                    "force_json",
                    "rpm",
                    "tpm",
                    "provider",
                    "extra_body",
                },
                exclude_none=True,
            )
            translator_args["glossary_generate_enable"] = payload.glossary_generate_enable
            translator_args["glossary_agent_config"] = build_glossary_agent_config()
            translator_config = DocxTranslatorConfig(**translator_args)
            translator_config.progress_tracker = progress_tracker

            html_exporter_config = Docx2HTMLExporterConfig(cdn=True)
            workflow_config = DocxWorkflowConfig(
                translator_config=translator_config,
                html_exporter_config=html_exporter_config,
                logger=task_logger,
                progress_tracker=progress_tracker,
            )
            return DocxWorkflow(config=workflow_config)

        elif isinstance(payload, SrtWorkflowParams):
            task_logger.info("构建 SrtWorkflow 配置。")
            translator_args = payload.model_dump(
                include={
                    "skip_translate",
                    "base_url",
                    "api_key",
                    "model_id",
                    "to_lang",
                    "custom_prompt",
                    "temperature",
                    "thinking",
                    "chunk_size",
                    "concurrent",
                    "insert_mode",
                    "separator",
                    "glossary_dict",
                    "timeout",
                    "retry",
                    "system_proxy_enable",
                    "force_json",
                    "rpm",
                    "tpm",
                    "provider",
                    "extra_body",
                },
                exclude_none=True,
            )
            translator_args["glossary_generate_enable"] = payload.glossary_generate_enable
            translator_args["glossary_agent_config"] = build_glossary_agent_config()
            translator_config = SrtTranslatorConfig(**translator_args)
            translator_config.progress_tracker = progress_tracker

            html_exporter_config = Srt2HTMLExporterConfig(cdn=True)
            workflow_config = SrtWorkflowConfig(
                translator_config=translator_config,
                html_exporter_config=html_exporter_config,
                logger=task_logger,
                progress_tracker=progress_tracker,
            )
            return SrtWorkflow(config=workflow_config)

        elif isinstance(payload, EpubWorkflowParams):
            task_logger.info("构建 EpubWorkflow 配置。")
            translator_args = payload.model_dump(
                include={
                    "skip_translate",
                    "base_url",
                    "api_key",
                    "model_id",
                    "to_lang",
                    "custom_prompt",
                    "temperature",
                    "thinking",
                    "chunk_size",
                    "concurrent",
                    "insert_mode",
                    "separator",
                    "glossary_dict",
                    "timeout",
                    "retry",
                    "system_proxy_enable",
                    "force_json",
                    "rpm",
                    "tpm",
                    "provider",
                    "extra_body",
                },
                exclude_none=True,
            )
            translator_args["glossary_generate_enable"] = payload.glossary_generate_enable
            translator_args["glossary_agent_config"] = build_glossary_agent_config()
            translator_config = EpubTranslatorConfig(**translator_args)
            translator_config.progress_tracker = progress_tracker

            html_exporter_config = Epub2HTMLExporterConfig(cdn=True)
            workflow_config = EpubWorkflowConfig(
                translator_config=translator_config,
                html_exporter_config=html_exporter_config,
                logger=task_logger,
                progress_tracker=progress_tracker,
            )
            return EpubWorkflow(config=workflow_config)

        elif isinstance(payload, HtmlWorkflowParams):
            task_logger.info("构建 HtmlWorkflow 配置。")
            translator_args = payload.model_dump(
                include={
                    "skip_translate",
                    "base_url",
                    "api_key",
                    "model_id",
                    "to_lang",
                    "custom_prompt",
                    "temperature",
                    "thinking",
                    "chunk_size",
                    "concurrent",
                    "insert_mode",
                    "separator",
                    "glossary_dict",
                    "timeout",
                    "retry",
                    "system_proxy_enable",
                    "force_json",
                    "rpm",
                    "tpm",
                    "provider",
                    "extra_body",
                },
                exclude_none=True,
            )
            translator_args["glossary_generate_enable"] = payload.glossary_generate_enable
            translator_args["glossary_agent_config"] = build_glossary_agent_config()
            translator_config = HtmlTranslatorConfig(**translator_args)
            translator_config.progress_tracker = progress_tracker

            workflow_config = HtmlWorkflowConfig(
                translator_config=translator_config, logger=task_logger,
                progress_tracker=progress_tracker,
            )
            return HtmlWorkflow(config=workflow_config)

        elif isinstance(payload, AssWorkflowParams):
            task_logger.info("构建 AssWorkflow 配置。")
            translator_args = payload.model_dump(
                include={
                    "skip_translate",
                    "base_url",
                    "api_key",
                    "model_id",
                    "to_lang",
                    "custom_prompt",
                    "temperature",
                    "thinking",
                    "chunk_size",
                    "concurrent",
                    "insert_mode",
                    "separator",
                    "glossary_dict",
                    "timeout",
                    "retry",
                    "system_proxy_enable",
                    "force_json",
                    "rpm",
                    "tpm",
                    "provider",
                    "extra_body",
                },
                exclude_none=True,
            )
            translator_args["glossary_generate_enable"] = payload.glossary_generate_enable
            translator_args["glossary_agent_config"] = build_glossary_agent_config()
            translator_config = AssTranslatorConfig(**translator_args)
            translator_config.progress_tracker = progress_tracker

            html_exporter_config = Ass2HTMLExporterConfig(cdn=True)
            workflow_config = AssWorkflowConfig(
                translator_config=translator_config,
                html_exporter_config=html_exporter_config,
                logger=task_logger,
                progress_tracker=progress_tracker,
            )
            return AssWorkflow(config=workflow_config)

        elif isinstance(payload, PPTXWorkflowParams):
            task_logger.info("构建 PPTXWorkflow 配置。")
            translator_args = payload.model_dump(
                include={
                    "skip_translate",
                    "base_url",
                    "api_key",
                    "model_id",
                    "to_lang",
                    "custom_prompt",
                    "temperature",
                    "thinking",
                    "chunk_size",
                    "concurrent",
                    "insert_mode",
                    "separator",
                    "glossary_dict",
                    "timeout",
                    "retry",
                    "system_proxy_enable",
                    "force_json",
                    "rpm",
                    "tpm",
                    "provider",
                    "extra_body",
                },
                exclude_none=True,
            )
            translator_args["glossary_generate_enable"] = payload.glossary_generate_enable
            translator_args["glossary_agent_config"] = build_glossary_agent_config()
            translator_config = PPTXTranslatorConfig(**translator_args)
            translator_config.progress_tracker = progress_tracker

            html_exporter_config = PPTX2HTMLExporterConfig(cdn=True)
            workflow_config = PPTXWorkflowConfig(
                translator_config=translator_config,
                html_exporter_config=html_exporter_config,
                logger=task_logger,
                progress_tracker=progress_tracker,
            )
            return PPTXWorkflow(config=workflow_config)

        else:
            raise TypeError(f"工作流类型 '{payload.workflow_type}' 的处理逻辑未实现。")

    def _build_export_map(
        self,
        workflow: Workflow,
        filename_stem: str,
        is_cdn_available: bool,
    ) -> Dict[str, Any]:
        """Build export map based on workflow capabilities."""
        export_map = {}

        if isinstance(workflow, MDFormatsExportable):
            export_map["markdown"] = (
                workflow.export_to_markdown,
                f"{filename_stem}_translated.md",
                True,
            )
            export_map["markdown_zip"] = (
                workflow.export_to_markdown_zip,
                f"{filename_stem}_translated.zip",
                False,
            )
        if isinstance(workflow, TXTExportable):
            export_map["txt"] = (
                workflow.export_to_txt,
                f"{filename_stem}_translated.txt",
                True,
            )
        if isinstance(workflow, JsonExportable):
            export_map["json"] = (
                workflow.export_to_json,
                f"{filename_stem}_translated.json",
                True,
            )
        if isinstance(workflow, XlsxExportable):
            export_map["xlsx"] = (
                workflow.export_to_xlsx,
                f"{filename_stem}_translated.xlsx",
                False,
            )
        if isinstance(workflow, CsvExportable):
            export_map["csv"] = (
                workflow.export_to_csv,
                f"{filename_stem}_translated.csv",
                False,
            )
        if isinstance(workflow, DocxExportable):
            # MarkdownBasedWorkflow needs md2docx_exporter_config
            if isinstance(workflow, MarkdownBasedWorkflow):
                if hasattr(workflow.config, 'md2docx_exporter_config') and workflow.config.md2docx_exporter_config is not None:
                    export_map["docx"] = (
                        workflow.export_to_docx,
                        f"{filename_stem}_translated.docx",
                        False,
                    )
            # DocxWorkflow can export docx directly
            elif isinstance(workflow, DocxWorkflow):
                export_map["docx"] = (
                    workflow.export_to_docx,
                    f"{filename_stem}_translated.docx",
                    False,
                )
        if isinstance(workflow, SrtExportable):
            export_map["srt"] = (
                workflow.export_to_srt,
                f"{filename_stem}_translated.srt",
                True,
            )
        if isinstance(workflow, EpubExportable):
            export_map["epub"] = (
                workflow.export_to_epub,
                f"{filename_stem}_translated.epub",
                False,
            )
        if isinstance(workflow, AssExportable):
            export_map["ass"] = (
                workflow.export_to_ass,
                f"{filename_stem}_translated.ass",
                True,
            )
        if isinstance(workflow, PPTXExportable):
            export_map["pptx"] = (
                workflow.export_to_pptx,
                f"{filename_stem}_translated.pptx",
                False,
            )

        if isinstance(workflow, HTMLExportable):
            html_config = None
            if isinstance(workflow, MarkdownBasedWorkflow):
                html_config = MD2HTMLExporterConfig(cdn=is_cdn_available)
            elif isinstance(workflow, TXTWorkflow):
                html_config = TXT2HTMLExporterConfig(cdn=is_cdn_available)
            elif isinstance(workflow, JsonWorkflow):
                html_config = Json2HTMLExporterConfig(cdn=is_cdn_available)
            elif isinstance(workflow, XlsxWorkflow):
                html_config = Xlsx2HTMLExporterConfig(cdn=is_cdn_available)
            elif isinstance(workflow, DocxWorkflow):
                html_config = Docx2HTMLExporterConfig(cdn=is_cdn_available)
            elif isinstance(workflow, SrtWorkflow):
                html_config = Srt2HTMLExporterConfig(cdn=is_cdn_available)
            elif isinstance(workflow, EpubWorkflow):
                html_config = Epub2HTMLExporterConfig(cdn=is_cdn_available)
            elif isinstance(workflow, AssWorkflow):
                html_config = Ass2HTMLExporterConfig(cdn=is_cdn_available)
            elif isinstance(workflow, PPTXWorkflow):
                html_config = PPTX2HTMLExporterConfig(cdn=is_cdn_available)
            export_map["html"] = (
                lambda: workflow.export_to_html(html_config),
                f"{filename_stem}_translated.html",
                True,
            )

        return export_map

    def cancel_task(self, task_id: str) -> Dict[str, Any]:
        """Cancel a running task."""
        task_state = self.tasks_state.get(task_id)
        if not task_state:
            raise HTTPException(status_code=404, detail=f"找不到任务ID '{task_id}'。")
        if (
            not task_state
            or not task_state["is_processing"]
            or not task_state["current_task_ref"]
        ):
            raise HTTPException(
                status_code=400, detail=f"任务ID '{task_id}' 没有正在进行的翻译任务可取消。"
            )

        task_to_cancel: Optional[asyncio.Task] = task_state["current_task_ref"]
        if not task_to_cancel or task_to_cancel.done():
            task_state["is_processing"] = False
            task_state["current_task_ref"] = None
            raise HTTPException(status_code=400, detail="任务已完成或已被取消。")

        print(f"[{task_id}] 收到取消翻译任务的请求。")
        task_to_cancel.cancel()
        task_state["status_message"] = "正在取消任务..."
        return {"cancelled": True, "message": "取消请求已发送。请等待状态更新。"}

    async def release_task(self, task_id: str) -> Dict[str, Any]:
        """Release task resources."""
        if task_id not in self.tasks_state:
            return {
                "released": False,
                "message": f"找不到任务ID '{task_id}'。"
            }
        task_state = self.tasks_state.get(task_id)
        message_parts = []
        if (
            task_state
            and task_state.get("is_processing")
            and task_state.get("current_task_ref")
        ):
            try:
                print(f"[{task_id}] 任务正在进行中，将在释放前尝试取消。")
                self.cancel_task(task_id)
                message_parts.append("任务已被取消。")
            except HTTPException as e:
                print(f"[{task_id}] 取消任务时出现预期中的情况（可能已完成）: {e.detail}")
                message_parts.append(f"任务取消步骤已跳过（可能已完成或取消）。")

        if task_state:
            temp_dir = task_state.get("temp_dir")
            if temp_dir and os.path.isdir(temp_dir):
                try:
                    shutil.rmtree(temp_dir)
                    message_parts.append("临时文件已清理。")
                    print(f"[{task_id}] 临时目录 '{temp_dir}' 已被删除。")
                except Exception as e:
                    message_parts.append(f"清理临时文件时出错: {e}。")
                    print(f"[{task_id}] 删除临时目录 '{temp_dir}' 时出错: {e}")

        self.tasks_state.pop(task_id, None)
        self.tasks_log_queues.pop(task_id, None)
        self.tasks_log_histories.pop(task_id, None)
        print(f"[{task_id}] 资源已成功释放。")
        message_parts.append(f"任务 '{task_id}' 的资源已释放。")
        return {"released": True, "message": " ".join(message_parts)}

    async def cleanup_all(self):
        """Cleanup all resources (called during application shutdown)."""
        pending_tasks = []
        for task_id, task_state in self.tasks_state.items():
            task_ref = task_state.get("current_task_ref")
            if task_ref and not task_ref.done():
                print(f"[{task_id}] 检测到未完成任务，正在强制取消...")
                task_ref.cancel()
                pending_tasks.append(task_ref)

        if pending_tasks:
            await asyncio.gather(*pending_tasks, return_exceptions=True)

        for task_id, task_state in self.tasks_state.items():
            temp_dir = task_state.get("temp_dir")
            if temp_dir and os.path.isdir(temp_dir):
                try:
                    shutil.rmtree(temp_dir, ignore_errors=True)
                    print(f"[{task_id}] 临时目录已清理: {temp_dir}")
                except Exception as e:
                    print(f"[{task_id}] 清理临时目录 '{temp_dir}' 时出错: {e}")


# Global singleton instance
_translation_service: Optional[TranslationService] = None


def get_translation_service() -> TranslationService:
    """Get the global translation service instance (singleton)."""
    global _translation_service
    if _translation_service is None:
        _translation_service = TranslationService()
    return _translation_service



# SPDX-FileCopyrightText: 2025 QinHan
# SPDX-License-Identifier: MPL-2.0
"""
DocuTranslate MCP Server

Model Context Protocol server for DocuTranslate, providing document translation
capabilities to AI assistants.

This implementation uses the shared TranslationService from the server layer
to ensure consistent task management between the Web backend and MCP server.
"""

import asyncio
import base64
import os
import sys
import tempfile
from datetime import datetime
from pathlib import Path
from typing import Any, Dict, Optional, List

# Shared server layer imports
from docutranslate.server import (
    TranslationService,
    get_translation_service,
)

from docutranslate import __version__
from docutranslate.translator import default_params
from docutranslate.core.schemas import TranslatePayload
from pydantic import TypeAdapter


# MCP Server configuration
SERVER_NAME = "docutranslate"
SERVER_VERSION = __version__


def _format_json(data: Any) -> str:
    """Format JSON for display"""
    import json
    return json.dumps(data, ensure_ascii=False, indent=2)


def _get_formats_info() -> Dict[str, Any]:
    """Get information about supported formats"""
    return {
        "input_formats": [
            "pdf", "docx", "doc", "xlsx", "xls", "csv", "md", "markdown",
            "txt", "json", "epub", "srt", "ass", "pptx", "ppt", "html", "htm",
            "png", "jpg"
        ],
        "output_formats_by_workflow": {
            "markdown_based": ["html", "markdown", "markdown_zip", "docx"],
            "txt": ["txt"],
            "json": ["json"],
            "xlsx": ["xlsx", "csv", "html"],
            "docx": ["docx", "html"],
            "srt": ["srt", "html"],
            "epub": ["epub", "html"],
            "html": ["html"],
            "ass": ["ass", "html"],
            "pptx": ["pptx", "html"],
        },
    }


def _task_state_to_dict(task_id: str, task_state: Optional[Dict[str, Any]]) -> Dict[str, Any]:
    """Convert task state to a serializable dict"""
    if not task_state:
        return {"task_id": task_id, "status": "not_found", "error": "Task not found"}

    status = "idle"
    if task_state.get("is_processing"):
        status = "running"
    elif task_state.get("error_flag"):
        status = "error"
    elif task_state.get("download_ready"):
        status = "completed"

    return {
        "task_id": task_id,
        "status": status,
        "status_message": task_state.get("status_message", ""),
        "progress_percent": task_state.get("progress_percent", 0),
        "download_ready": task_state.get("download_ready", False),
        "downloadable_files": task_state.get("downloadable_files", {}),
        "attachment_files": task_state.get("attachment_files", {}),
    }


# ===========================================================================
# Conditional MCP functionality - only available if mcp dependencies are installed
# ===========================================================================

try:
    from mcp.server.fastmcp import FastMCP, Context
    MCP_AVAILABLE = True
except ImportError:
    MCP_AVAILABLE = False
    FastMCP = None
    Context = None


if MCP_AVAILABLE and FastMCP is not None and Context is not None:

    def create_mcp_server(
        host: str = "127.0.0.1",
        port: int = 8000,
        translation_service: Optional[TranslationService] = None,
    ):
        """Create and configure the FastMCP server

        Args:
            host: Host address for SSE/streamable-http transport
            port: Port number for SSE/streamable-http transport
            translation_service: Optional TranslationService instance (uses global if not provided)

        Returns:
            Configured FastMCP server
        """
        # Use provided translation service or get the global one
        service = translation_service or get_translation_service()

        # Client configuration - load from environment variables first
        client_config = {
            "api_key": os.environ.get("DOCUTRANSLATE_API_KEY", ""),
            "base_url": os.environ.get("DOCUTRANSLATE_BASE_URL", ""),
            "model_id": os.environ.get("DOCUTRANSLATE_MODEL_ID", ""),
            "to_lang": os.environ.get("DOCUTRANSLATE_TO_LANG", "中文"),
            "concurrent": int(os.environ.get("DOCUTRANSLATE_CONCURRENT", "10")),
            "convert_engine": os.environ.get("DOCUTRANSLATE_CONVERT_ENGINE", ""),
            "mineru_token": os.environ.get("DOCUTRANSLATE_MINERU_TOKEN", ""),
        }

        # Create FastMCP instance
        mcp = FastMCP(
            name=SERVER_NAME,
            instructions="DocuTranslate MCP server for document translation. "
                         "Step 1: Use submit_task to start a translation task, "
                         "Step 2: Use get_task_status to check progress - when completed, "
                         "it will show all available formats and attachments, "
                         "Step 3: Use download_file to save translations or attachments, "
                         "Step 4: Use release_task to clean up resources when done. "
                         "The translation uses the full workflow engine from DocuTranslate.",
            host=host,
            port=port,
        )

        @mcp.tool()
        async def get_status() -> str:
            """Get current server status and configuration information."""
            is_configured = bool(client_config.get("api_key") and client_config.get("base_url") and client_config.get("model_id"))
            status_info = {
                "server": "docutranslate",
                "version": __version__,
                "status": "ready",
                "client_configured": is_configured,
                "active_tasks": len(service.tasks_state),
            }
            return f"Server Status:\n{_format_json(status_info)}"

        @mcp.tool()
        async def configure_client(
            api_key: Optional[str] = None,
            base_url: Optional[str] = None,
            model_id: Optional[str] = None,
            to_lang: Optional[str] = None,
            concurrent: Optional[int] = None,
            convert_engine: Optional[str] = None,
            mineru_token: Optional[str] = None,
        ) -> str:
            """Configure the DocuTranslate client LLM settings.
            If already configured via environment variables, this tool can override them.

            Args:
                api_key: AI platform API key
                base_url: AI platform base URL
                model_id: Model ID to use
                to_lang: Target language (default: 中文)
                concurrent: Number of concurrent requests (default: 10)
                convert_engine: PDF conversion engine
                mineru_token: MinerU API token
            """
            if api_key is not None:
                client_config["api_key"] = api_key
            if base_url is not None:
                client_config["base_url"] = base_url
            if model_id is not None:
                client_config["model_id"] = model_id
            if to_lang is not None:
                client_config["to_lang"] = to_lang
            if concurrent is not None:
                client_config["concurrent"] = concurrent
            if convert_engine is not None:
                client_config["convert_engine"] = convert_engine
            if mineru_token is not None:
                client_config["mineru_token"] = mineru_token

            # Check if we have the required config
            has_required = bool(client_config.get("api_key") and client_config.get("base_url") and client_config.get("model_id"))

            if has_required:
                return (
                    "Client configuration updated successfully.\n"
                    "You can now use submit_task without providing api_key, base_url, and model_id every time."
                )
            else:
                missing = []
                if not client_config.get("api_key"):
                    missing.append("api_key")
                if not client_config.get("base_url"):
                    missing.append("base_url")
                if not client_config.get("model_id"):
                    missing.append("model_id")
                return (
                    f"Configuration updated, but missing required fields: {', '.join(missing)}\n"
                    "Please provide all required fields to use the translation service."
                )

        @mcp.tool()
        async def get_client_config() -> str:
            """Get current client configuration (without sensitive data like API keys)."""
            # Mask sensitive data
            masked_config = client_config.copy()
            if masked_config.get("api_key"):
                key = masked_config["api_key"]
                masked_config["api_key"] = key[:4] + "..." + key[-4:] if len(key) > 8 else "****"
            if masked_config.get("mineru_token"):
                token = masked_config["mineru_token"]
                masked_config["mineru_token"] = token[:4] + "..." + token[-4:] if len(token) > 8 else "****"
            return f"Current Client Configuration:\n{_format_json(masked_config)}"

        @mcp.tool()
        async def submit_task(
            file_path: str,
            api_key: str = "",
            base_url: str = "",
            model_id: str = "",
            to_lang: Optional[str] = None,
            workflow_type: str = "auto",
            skip_translate: bool = False,
            glossary_generate_enable: bool = False,
            glossary_dict_json: str = "",
            glossary_agent_config_json: str = "",
            extra_body_json: str = "",
            # LLM 进阶参数
            chunk_size: Optional[int] = None,
            concurrent: Optional[int] = None,
            temperature: Optional[float] = None,
            timeout: Optional[int] = None,
            thinking: Optional[str] = None,
            retry: Optional[int] = None,
            system_proxy_enable: Optional[bool] = None,
            custom_prompt: str = "",
            force_json: Optional[bool] = None,
            rpm: Optional[int] = None,
            tpm: Optional[int] = None,
            provider: Optional[str] = None,
            # Markdown 工作流参数
            convert_engine: Optional[str] = None,
            md2docx_engine: Optional[str] = None,
            mineru_token: str = "",
            model_version: Optional[str] = None,
            formula_ocr: Optional[bool] = None,
            code_ocr: Optional[bool] = None,
            mineru_deploy_base_url: Optional[str] = None,
            mineru_deploy_backend: Optional[str] = None,
            mineru_deploy_parse_method: Optional[str] = None,
            mineru_deploy_table_enable: Optional[bool] = None,
            mineru_deploy_formula_enable: Optional[bool] = None,
            mineru_deploy_start_page_id: Optional[int] = None,
            mineru_deploy_end_page_id: Optional[int] = None,
            mineru_deploy_lang_list: Optional[List[str]] = None,
            mineru_deploy_server_url: str = "",
            # 其他工作流参数
            insert_mode: Optional[str] = None,
            separator: Optional[str] = None,
            segment_mode: Optional[str] = None,
            translate_regions: Optional[List[str]] = None,
            json_paths: Optional[List[str]] = None,
        ) -> str:
            """Submit a translation task (asynchronous, returns immediately).
            Use get_task_status to check progress. When complete, it will show
            all available formats and attachments. Use download_file to save files.
            Supports PDF, DOCX, XLSX, MD, TXT, JSON, EPUB, SRT, ASS, PPTX, HTML formats.

            Args:
                file_path: Path to the file to translate
                api_key: AI platform API key
                base_url: AI platform base URL
                model_id: Model ID to use
                to_lang: Target language (default: 中文)
                workflow_type: Workflow type (auto-detected if not specified)
                skip_translate: Skip translation, only parse the document
                glossary_generate_enable: Enable automatic glossary generation
                glossary_dict_json: Glossary dictionary JSON string, format: {"原文":"译文"}
                glossary_agent_config_json: Glossary agent config JSON string (contains base_url, model_id, etc.)
                extra_body_json: Extra request body parameters JSON string, will be merged into API request
                chunk_size: Text chunk size for translation
                concurrent: Number of concurrent requests
                temperature: LLM temperature parameter
                timeout: Request timeout in seconds
                thinking: Thinking mode (default, enable, disable)
                retry: Number of retries for failed chunks
                system_proxy_enable: Enable system proxy
                custom_prompt: Custom system prompt
                force_json: Force JSON output
                rpm: RPM limit (Requests Per Minute)
                tpm: TPM limit (Tokens Per Minute)
                provider: LLM provider identifier
                convert_engine: PDF conversion engine (identity, mineru, docling, mineru_deploy)
                md2docx_engine: Markdown to docx engine (python, pandoc, auto)
                mineru_token: MinerU Cloud API Token
                model_version: MinerU Cloud model version (pipeline, vlm)
                formula_ocr: Enable formula OCR
                code_ocr: Enable code block OCR
                mineru_deploy_base_url: MinerU local deployment base URL
                mineru_deploy_backend: MinerU local backend type
                mineru_deploy_parse_method: MinerU parse method (auto, txt, ocr)
                mineru_deploy_table_enable: Enable table parsing
                mineru_deploy_formula_enable: Enable formula parsing
                mineru_deploy_start_page_id: Start page ID
                mineru_deploy_end_page_id: End page ID
                mineru_deploy_lang_list: Language list
                mineru_deploy_server_url: MinerU Server URL
                insert_mode: Insert mode (replace, append, prepend)
                separator: Separator for append/prepend
                segment_mode: [Txt only] Segment mode (line, paragraph, none)
                translate_regions: [Xlsx only] Translation regions list
                json_paths: [Json only] JsonPath expressions list
            """
            if not os.path.exists(file_path):
                return f"Error: File not found: {file_path}"

            # Read the file
            try:
                with open(file_path, "rb") as f:
                    file_contents = f.read()
            except Exception as e:
                return f"Error reading file: {e}"

            import json

            # Parse glossary dict if provided
            parsed_glossary_dict = None
            if glossary_dict_json and glossary_dict_json.strip():
                try:
                    parsed_glossary_dict = json.loads(glossary_dict_json)
                    if not isinstance(parsed_glossary_dict, dict):
                        return "Error: glossary_dict_json must be a dictionary"
                except Exception as e:
                    return f"Error parsing glossary_dict_json: {e}"

            # Parse glossary agent config if provided
            parsed_glossary_agent = None
            if glossary_agent_config_json and glossary_agent_config_json.strip():
                try:
                    parsed_glossary_agent = json.loads(glossary_agent_config_json)
                except Exception as e:
                    return f"Error parsing glossary_agent_config_json: {e}"

            # Parse extra_body if provided
            if extra_body_json and extra_body_json.strip():
                try:
                    parsed_extra = json.loads(extra_body_json)
                    if not isinstance(parsed_extra, dict):
                        return "Error: extra_body_json must be a JSON object"
                except Exception as e:
                    return f"Error parsing extra_body_json: {e}"

            # Build payload dict - use AutoWorkflowParams with extra=allow
            # This avoids validation errors for workflow-specific optional fields
            # Use client_config for defaults if parameters not provided
            payload_dict = {
                "workflow_type": workflow_type,
                "to_lang": to_lang or client_config["to_lang"] or "中文",
                "skip_translate": skip_translate,
                "glossary_generate_enable": glossary_generate_enable,
                "glossary_dict": parsed_glossary_dict,
                "glossary_agent_config": parsed_glossary_agent,
            }
            # Add extra_body if provided
            if extra_body_json and extra_body_json.strip():
                payload_dict["extra_body"] = extra_body_json

            # Add AI config - use parameter if provided, otherwise use client_config
            use_api_key = api_key or client_config["api_key"]
            use_base_url = base_url or client_config["base_url"]
            use_model_id = model_id or client_config["model_id"]

            if use_api_key:
                payload_dict["api_key"] = use_api_key
            if use_base_url:
                payload_dict["base_url"] = use_base_url
            if use_model_id:
                payload_dict["model_id"] = use_model_id

            # Add LLM advanced parameters if provided
            if chunk_size is not None:
                payload_dict["chunk_size"] = chunk_size
            use_concurrent = concurrent if concurrent is not None else client_config["concurrent"]
            if use_concurrent is not None:
                payload_dict["concurrent"] = use_concurrent
            if temperature is not None:
                payload_dict["temperature"] = temperature
            if timeout is not None:
                payload_dict["timeout"] = timeout
            if thinking is not None:
                payload_dict["thinking"] = thinking
            if retry is not None:
                payload_dict["retry"] = retry
            if system_proxy_enable is not None:
                payload_dict["system_proxy_enable"] = system_proxy_enable
            if custom_prompt:
                payload_dict["custom_prompt"] = custom_prompt
            if force_json is not None:
                payload_dict["force_json"] = force_json
            if rpm is not None:
                payload_dict["rpm"] = rpm
            if tpm is not None:
                payload_dict["tpm"] = tpm
            if provider:
                payload_dict["provider"] = provider

            # Add Markdown workflow parameters if provided
            use_convert_engine = convert_engine or client_config["convert_engine"]
            if use_convert_engine:
                payload_dict["convert_engine"] = use_convert_engine
            if md2docx_engine:
                payload_dict["md2docx_engine"] = md2docx_engine
            use_mineru_token = mineru_token or client_config["mineru_token"]
            if use_mineru_token:
                payload_dict["mineru_token"] = use_mineru_token
            if model_version:
                payload_dict["model_version"] = model_version
            if formula_ocr is not None:
                payload_dict["formula_ocr"] = formula_ocr
            if code_ocr is not None:
                payload_dict["code_ocr"] = code_ocr
            if mineru_deploy_base_url:
                payload_dict["mineru_deploy_base_url"] = mineru_deploy_base_url
            if mineru_deploy_backend:
                payload_dict["mineru_deploy_backend"] = mineru_deploy_backend
            if mineru_deploy_parse_method:
                payload_dict["mineru_deploy_parse_method"] = mineru_deploy_parse_method
            if mineru_deploy_table_enable is not None:
                payload_dict["mineru_deploy_table_enable"] = mineru_deploy_table_enable
            if mineru_deploy_formula_enable is not None:
                payload_dict["mineru_deploy_formula_enable"] = mineru_deploy_formula_enable
            if mineru_deploy_start_page_id is not None:
                payload_dict["mineru_deploy_start_page_id"] = mineru_deploy_start_page_id
            if mineru_deploy_end_page_id is not None:
                payload_dict["mineru_deploy_end_page_id"] = mineru_deploy_end_page_id
            if mineru_deploy_lang_list:
                payload_dict["mineru_deploy_lang_list"] = mineru_deploy_lang_list
            if mineru_deploy_server_url:
                payload_dict["mineru_deploy_server_url"] = mineru_deploy_server_url

            # Add other workflow parameters if provided
            if insert_mode:
                payload_dict["insert_mode"] = insert_mode
            if separator:
                payload_dict["separator"] = separator
            if segment_mode:
                payload_dict["segment_mode"] = segment_mode
            if translate_regions:
                payload_dict["translate_regions"] = translate_regions
            if json_paths:
                payload_dict["json_paths"] = json_paths

            try:
                # Validate and create payload - AutoWorkflowParams allows extra fields
                payload = TypeAdapter(TranslatePayload).validate_python(payload_dict)
            except Exception as e:
                return f"Error validating parameters: {e}"

            # Create task_id and start translation
            task_id = os.path.basename(file_path)[:8] + "_" + base64.urlsafe_b64encode(os.urandom(4)).decode()[:8]

            try:
                # Store original file path for reference
                if not hasattr(service, "_mcp_output_options"):
                    service._mcp_output_options = {}
                service._mcp_output_options[task_id] = {
                    "original_file_path": file_path,
                }

                response = await service.start_translation(
                    task_id=task_id,
                    payload=payload,
                    file_contents=file_contents,
                    original_filename=os.path.basename(file_path),
                )
                return f"Translation task submitted successfully.\ntask_id: {task_id}\n\nUse get_task_status to check progress. When completed, it will show all available formats for download."
            except Exception as e:
                return f"Error starting translation: {e}"

        @mcp.tool()
        async def get_task_status(task_id: str, wait_seconds: float = 0) -> str:
            """Get status of a translation task.
            When task is completed, returns all available formats and attachments for download.
            Use download_file to save files to local.

            Args:
                task_id: The task ID from submit_task
                wait_seconds: Maximum seconds to wait for status change before returning (0 = return immediately)
            """
            task_state = service.get_task_state(task_id)

            if wait_seconds > 0 and task_state:
                # Wait for status change or timeout
                initial_processing = task_state.get("is_processing")
                initial_progress = task_state.get("progress_percent", 0)
                deadline = asyncio.get_event_loop().time() + wait_seconds

                while asyncio.get_event_loop().time() < deadline:
                    await asyncio.sleep(0.2)
                    task_state = service.get_task_state(task_id)
                    if not task_state:
                        break
                    if task_state.get("is_processing") != initial_processing or task_state.get("progress_percent", 0) != initial_progress:
                        break

            status_dict = _task_state_to_dict(task_id, task_state)
            status = status_dict["status"]

            # If completed, show user-friendly format with download info
            if status == "completed":
                response_parts = [
                    "Translation completed successfully!",
                    f"Task ID: {task_id}",
                    "",
                    "Available formats:",
                ]

                # Get output options for original file info
                output_options = {}
                if hasattr(service, "_mcp_output_options"):
                    output_options = service._mcp_output_options.get(task_id, {})
                original_file_path = output_options.get("original_file_path", "")
                if original_file_path:
                    response_parts.insert(1, f"Input: {original_file_path}")

                downloadable_files = status_dict.get("downloadable_files", {})
                for fmt, info in downloadable_files.items():
                    response_parts.append(f"  - {fmt}: {info['filename']}")

                attachment_files = status_dict.get("attachment_files", {})
                if attachment_files:
                    response_parts.append("")
                    response_parts.append("Attachments (e.g., glossary):")
                    for name, info in attachment_files.items():
                        response_parts.append(f"  - {name}: {info['filename']}")

                response_parts.append("")
                response_parts.append("Use download_file to save translations or attachments to your local file system.")
                response_parts.append("After downloading all needed files, use release_task to clean up temporary files.")
                response_parts.append("")
                response_parts.append("Full status details:")
                response_parts.append(_format_json(status_dict))

                return "\n".join(response_parts)

            # For other statuses, just return the JSON
            return f"Task Status:\n{_format_json(status_dict)}"

        @mcp.tool()
        async def download_file(
            task_id: str,
            file_name: str,
            output_dir: str = "./output",
            output_name: Optional[str] = None,
        ) -> str:
            """Download a translated file or attachment to local file system.

            Args:
                task_id: The task ID from submit_task
                file_name: The format name (e.g., 'docx', 'html') or attachment name (e.g., 'glossary') to download
                output_dir: Output directory for saving the file (default: ./output)
                output_name: Output filename (optional, auto-generated if not provided)
            """
            task_state = service.get_task_state(task_id)
            if not task_state:
                return f"Error: Task not found: {task_id}"

            if not task_state.get("download_ready"):
                return "Error: No result available yet"

            # Get downloadable files and attachments
            downloadable_files = task_state.get("downloadable_files", {})
            attachment_files = task_state.get("attachment_files", {})

            # Find the file - check both downloadable and attachments
            all_files = {**downloadable_files, **attachment_files}

            if file_name not in all_files:
                available = list(all_files.keys())
                return f"Error: File '{file_name}' not available. Available: {available}"

            file_info = all_files[file_name]
            temp_file_path = file_info["path"]
            filename = file_info["filename"]

            # Read the file
            try:
                with open(temp_file_path, "rb") as f:
                    content_bytes = f.read()
            except Exception as e:
                return f"Error reading result file: {e}"

            # Save to file
            try:
                os.makedirs(output_dir, exist_ok=True)
                if output_name:
                    save_filename = output_name
                else:
                    save_filename = filename
                saved_path = os.path.join(output_dir, save_filename)
                with open(saved_path, "wb") as f:
                    f.write(content_bytes)
            except Exception as e:
                return f"Error saving file: {e}"

            return f"File saved successfully:\nFile: {file_name}\nSaved to: {saved_path}"

        @mcp.tool()
        async def cancel_task(task_id: str) -> str:
            """Cancel a pending or running task.

            Args:
                task_id: The task ID to cancel
            """
            try:
                result = service.cancel_task(task_id)
                return f"Task {task_id} cancelled successfully"
            except Exception as e:
                return f"Error: {e}"

        @mcp.tool()
        async def release_task(task_id: str) -> str:
            """Release task resources (temp files, memory, etc.) after translation is complete
            and all files have been downloaded.

            Args:
                task_id: The task ID to release
            """
            try:
                result = await service.release_task(task_id)
                return f"Task {task_id} released successfully. {result['message']}"
            except Exception as e:
                return f"Error releasing task: {e}"

        @mcp.tool()
        async def load_glossary_file(file_path: str) -> str:
            """Load a glossary file (JSON or CSV) and return it as a JSON string for use in submit_task.

            Supports:
            - JSON files with format: {"原文": "译文", "Original": "Translated"}
            - CSV files with two columns: first column = original, second column = translated

            Args:
                file_path: Path to the glossary file (.json or .csv)
            """
            if not os.path.exists(file_path):
                return f"Error: File not found: {file_path}"

            import json

            ext = os.path.splitext(file_path)[1].lower()

            try:
                if ext == ".json":
                    with open(file_path, "r", encoding="utf-8") as f:
                        glossary_dict = json.load(f)
                    if not isinstance(glossary_dict, dict):
                        return "Error: JSON file must contain a dictionary"

                elif ext == ".csv":
                    import csv
                    glossary_dict = {}
                    with open(file_path, "r", encoding="utf-8") as f:
                        reader = csv.reader(f)
                        for row in reader:
                            if len(row) >= 2:
                                key = row[0].strip()
                                value = row[1].strip()
                                if key and value:
                                    glossary_dict[key] = value

                else:
                    return f"Error: Unsupported file format: {ext}. Use .json or .csv"

                glossary_json = json.dumps(glossary_dict, ensure_ascii=False)
                return f"Glossary loaded successfully ({len(glossary_dict)} entries).\n\nUse this in submit_task:\nglossary_dict_json='''{glossary_json}'''"

            except Exception as e:
                return f"Error loading glossary file: {e}"

        @mcp.tool()
        async def get_supported_formats() -> str:
            """Get list of supported file formats and output options."""
            formats_info = _get_formats_info()
            return f"Supported formats:\n{_format_json(formats_info)}"

        @mcp.resource("docutranslate://info", name="DocuTranslate Server Information")
        async def get_info_resource() -> str:
            """Information about the DocuTranslate MCP server"""
            status_info = {
                "server": "docutranslate",
                "version": __version__,
                "status": "ready",
            }
            return _format_json(status_info)

        @mcp.resource("docutranslate://formats", name="Supported Formats")
        async def get_formats_resource() -> str:
            """List of supported file formats"""
            return _format_json(_get_formats_info())

        return mcp


    def get_sse_app(
        translation_service: Optional[TranslationService] = None,
        host: str = "127.0.0.1",
        port: int = 8000,
        enable_cors: bool = False,
        allow_origin_regex: str = r"^(https?://.*|null|file://.*)$",
    ):
        """Get the SSE Starlette app for mounting to existing FastAPI.

        Args:
            translation_service: Optional TranslationService instance (uses global if not provided)
            host: Host address (optional, not used when mounted)
            port: Port number (optional, not used when mounted)
            enable_cors: Whether to enable CORS for the SSE app
            allow_origin_regex: Regex for allowed CORS origins

        Returns:
            Starlette application that can be mounted to FastAPI
        """
        mcp = create_mcp_server(host=host, port=port, translation_service=translation_service)
        sse_app = mcp.sse_app()

        if enable_cors:
            from starlette.middleware.cors import CORSMiddleware
            sse_app.add_middleware(
                CORSMiddleware,
                allow_origin_regex=allow_origin_regex,
                allow_credentials=True,
                allow_methods=["*"],
                allow_headers=["*"],
            )

        return sse_app


    def run_mcp_server(
        transport: str = "stdio",
        host: str = "127.0.0.1",
        port: int = 8000,
        translation_service: Optional[TranslationService] = None,
    ):
        """Run the MCP server (entry point)

        Args:
            transport: Transport protocol to use ("stdio", "sse", or "streamable-http")
            host: Host address for SSE/streamable-http transport
            port: Port number for SSE/streamable-http transport
            translation_service: Optional TranslationService instance (uses global if not provided)
        """
        # Initialize the translation service with a dummy HTTP client for standalone mode
        service = translation_service or get_translation_service()
        if service.httpx_client is None:
            import httpx

            async def init_service():
                service.httpx_client = httpx.AsyncClient()
                service.main_event_loop = asyncio.get_running_loop()

            try:
                loop = asyncio.get_running_loop()
                loop.create_task(init_service())
            except RuntimeError:
                # No loop running yet, will be initialized when needed
                pass

        mcp = create_mcp_server(host=host, port=port, translation_service=service)

        # For SSE transport, mount at /mcp prefix
        if transport in ("sse", "streamable-http"):
            # Create a wrapper app with /mcp prefix
            from starlette.applications import Starlette

            wrapper_app = Starlette()
            sse_app = mcp.sse_app()
            wrapper_app.mount("/mcp", sse_app, name="mcp")

            # Run the wrapper app with uvicorn
            import uvicorn
            print(f"Starting MCP SSE server at http://{host}:{port}/mcp/sse")
            uvicorn.run(wrapper_app, host=host, port=port)
        else:
            # Run stdio transport normally
            mcp.run(transport=transport)


    def main():
        """Main entry point for module execution"""
        import argparse

        parser = argparse.ArgumentParser(
            description="DocuTranslate MCP Server"
        )
        parser.add_argument(
            "--transport",
            type=str,
            default="stdio",
            choices=["stdio", "sse", "streamable-http"],
            help="Transport protocol to use (default: stdio)"
        )
        parser.add_argument(
            "--host",
            type=str,
            default="127.0.0.1",
            help="Host address for SSE/streamable-http transport (default: 127.0.0.1)"
        )
        parser.add_argument(
            "--port",
            type=int,
            default=8000,
            help="Port number for SSE/streamable-http transport (default: 8000)"
        )

        args = parser.parse_args()
        run_mcp_server(
            transport=args.transport,
            host=args.host,
            port=args.port
        )


    if __name__ == "__main__":
        main()


else:
    # MCP dependencies not installed - define placeholder functions
    MCP_AVAILABLE = False

    def _mcp_not_available(*args, **kwargs):
        raise ImportError(
            "MCP dependencies not installed. "
            "Install with: pip install docutranslate[mcp]"
        )

    create_mcp_server = _mcp_not_available
    get_sse_app = _mcp_not_available
    run_mcp_server = _mcp_not_available



# SPDX-FileCopyrightText: 2025 QinHan
# SPDX-License-Identifier: MPL-2.0
import asyncio
from dataclasses import dataclass
from io import BytesIO
from typing import Self, Literal, List, Optional, Dict, Tuple, Set
import zipfile
import re
import xml.etree.ElementTree as ET

from openpyxl.utils.cell import coordinate_to_tuple, range_boundaries

from docutranslate.agents.segments_agent import SegmentsTranslateAgentConfig, SegmentsTranslateAgent
from docutranslate.ir.document import Document
from docutranslate.translator.ai_translator.base import AiTranslatorConfig, AiTranslator


@dataclass
class XlsxTranslatorConfig(AiTranslatorConfig):
    insert_mode: Literal["replace", "append", "prepend"] = "replace"
    separator: str = "\n"
    # 指定翻译区域列表。
    translate_regions: Optional[List[str]] = None


class XlsxTranslator(AiTranslator):
    def __init__(self, config: XlsxTranslatorConfig):
        super().__init__(config=config)
        self.chunk_size = config.chunk_size
        self.translate_agent = None
        glossary_dict = self.glossary.glossary_dict if self.glossary else None
        if not self.skip_translate:
            agent_config = SegmentsTranslateAgentConfig(
                custom_prompt=config.custom_prompt,
                to_lang=config.to_lang,
                base_url=config.base_url,
                api_key=config.api_key,
                model_id=config.model_id,
                temperature=config.temperature,
                thinking=config.thinking,
                concurrent=config.concurrent,
                timeout=config.timeout,
                logger=self.logger,
                glossary_dict=glossary_dict,
                retry=config.retry,
                system_proxy_enable=config.system_proxy_enable,
                force_json=config.force_json,
                rpm=config.rpm,
                tpm=config.tpm,
                provider=config.provider,
                extra_body=config.extra_body,
            )
            self.translate_agent = SegmentsTranslateAgent(agent_config)
        self.insert_mode = config.insert_mode
        self.separator = config.separator
        self.translate_regions = config.translate_regions

        # 注册常用命名空间，防止ElementTree写回时产生 ns0, ns1 等前缀导致Excel报错
        self.NS_MAIN = 'http://schemas.openxmlformats.org/spreadsheetml/2006/main'
        self.NS_REL = 'http://schemas.openxmlformats.org/officeDocument/2006/relationships'
        self.NS_MC = 'http://schemas.openxmlformats.org/markup-compatibility/2006'
        self.NS_XML = 'http://www.w3.org/XML/1998/namespace'

        # Drawing / VML / Comments namespaces (保留这些注册以防止绘图报错)
        self.NS_XDR = 'http://schemas.openxmlformats.org/drawingml/2006/spreadsheetDrawing'
        self.NS_A = 'http://schemas.openxmlformats.org/drawingml/2006/main'
        self.NS_VML = 'urn:schemas-microsoft-com:vml'

        ET.register_namespace('', self.NS_MAIN)
        ET.register_namespace('r', self.NS_REL)
        ET.register_namespace('mc', self.NS_MC)
        ET.register_namespace('xdr', self.NS_XDR)
        ET.register_namespace('a', self.NS_A)
        ET.register_namespace('v', self.NS_VML)
        ET.register_namespace('x14ac', "http://schemas.microsoft.com/office/spreadsheetml/2009/9/ac")
        ET.register_namespace('x15', "http://schemas.microsoft.com/office/spreadsheetml/2010/11/main")
        ET.register_namespace('x15ac', "http://schemas.microsoft.com/office/spreadsheetml/2010/11/ac")

    # =========================================================================
    # 核心辅助方法
    # =========================================================================

    def _tag_is(self, elem: ET.Element, tag_name: str) -> bool:
        """判断元素的标签名是否匹配（忽略命名空间）。"""
        return elem.tag.endswith(f"}}{tag_name}") or elem.tag == tag_name

    def _find_child(self, parent: ET.Element, tag_name: str) -> Optional[ET.Element]:
        """在直接子节点中查找指定标签（忽略命名空间）。"""
        for child in parent:
            if self._tag_is(child, tag_name):
                return child
        return None

    def _sanitize_xml_text(self, text: str) -> str:
        """移除 Excel XML 不允许的控制字符。"""
        if not text:
            return ""
        # 移除 ASCII 0-8, 11-12, 14-31
        return re.sub(r'[\x00-\x08\x0b\x0c\x0e-\x1f]', '', text)

    def _get_child_text(self, parent: ET.Element, tag_name: str) -> Optional[str]:
        child = self._find_child(parent, tag_name)
        return child.text if child is not None else None

    # =========================================================================
    # 辅助逻辑
    # =========================================================================

    def _get_shared_strings(self, zf: zipfile.ZipFile) -> List[str]:
        if "xl/sharedStrings.xml" not in zf.namelist():
            return []
        shared_strings = []
        with zf.open("xl/sharedStrings.xml") as f:
            context = ET.iterparse(f, events=("end",))
            for event, elem in context:
                if self._tag_is(elem, "si"):
                    texts = []
                    for child in elem.iter():
                        if self._tag_is(child, "t") and child.text:
                            texts.append(child.text)
                    shared_strings.append("".join(texts))
                    elem.clear()
        return shared_strings

    def _get_sheet_mapping(self, zf: zipfile.ZipFile) -> Dict[str, str]:
        sheet_name_to_rid = {}
        try:
            with zf.open("xl/workbook.xml") as f:
                root = ET.fromstring(f.read())
                for sheet in root.iter():
                    if self._tag_is(sheet, "sheet"):
                        name = sheet.get("name")
                        rid = None
                        for k, v in sheet.attrib.items():
                            if k.endswith("id"):
                                rid = v
                                break
                        if name and rid:
                            sheet_name_to_rid[name] = rid
        except Exception:
            return {}

        rid_to_target = {}
        try:
            with zf.open("xl/_rels/workbook.xml.rels") as f:
                root = ET.fromstring(f.read())
                for child in root:
                    rid = child.get("Id")
                    target = child.get("Target")
                    if rid and target:
                        target = target.replace("\\", "/")
                        if target.startswith("/"):
                            target = target.lstrip("/")
                        else:
                            target = "xl/" + target
                        rid_to_target[rid] = target
        except Exception:
            return {}

        mapping = {}
        for name, rid in sheet_name_to_rid.items():
            if rid in rid_to_target:
                mapping[name] = rid_to_target[rid]
        return mapping

    def _normalize_range(self, range_str: str) -> str:
        if re.match(r"^[A-Za-z]+$", range_str):
            return f"{range_str}:{range_str}"
        if re.match(r"^\d+$", range_str):
            return f"{range_str}:{range_str}"
        return range_str

    def _parse_region_boundaries(self, sheet_mapping: Dict[str, str]) -> Dict[str, List[Tuple]]:
        if not self.translate_regions:
            return {}
        region_map = {}
        global_regions = []

        for region in self.translate_regions:
            sheet_name = None
            raw_range = region.strip()
            if "!" in raw_range:
                parts = raw_range.split("!", 1)
                sheet_name = parts[0].strip("'")
                range_part = self._normalize_range(parts[1])
            else:
                range_part = self._normalize_range(raw_range)
            try:
                boundaries = range_boundaries(range_part)
                if sheet_name:
                    filename = sheet_mapping.get(sheet_name)
                    if filename:
                        if filename not in region_map:
                            region_map[filename] = []
                        region_map[filename].append(boundaries)
                else:
                    global_regions.append(boundaries)
            except Exception as e:
                self.logger.warning(f"无法解析区域 '{region}': {e}")

        if global_regions:
            target_files = set(sheet_mapping.values())
            for f in target_files:
                if f not in region_map:
                    region_map[f] = []
                region_map[f].extend(global_regions)
        return region_map

    def _is_in_boundaries(self, col: int, row: int, boundaries_list: List[Tuple]) -> bool:
        for (min_col, min_row, max_col, max_row) in boundaries_list:
            if min_col is not None and col < min_col: continue
            if min_row is not None and row < min_row: continue
            if max_col is not None and col > max_col: continue
            if max_row is not None and row > max_row: continue
            return True
        return False

    def _apply_insert_mode(self, original: str, translated: str) -> str:
        translated = self._sanitize_xml_text(translated)
        if self.insert_mode == "append":
            return f"{original}{self.separator}{translated}"
        elif self.insert_mode == "prepend":
            return f"{translated}{self.separator}{original}"
        else:
            return translated

    # =========================================================================
    # 区域处理
    # =========================================================================

    def _get_texts_xml_regions(self, document: Document) -> List[str]:
        texts_to_translate = set()
        with zipfile.ZipFile(BytesIO(document.content), 'r') as zf:
            shared_strings = self._get_shared_strings(zf)
            sheet_mapping = self._get_sheet_mapping(zf)
            if not sheet_mapping:
                all_sheets = [n for n in zf.namelist() if n.startswith("xl/worksheets/sheet") and n.endswith(".xml")]
                for s in all_sheets:
                    sheet_mapping[f"Unknown_{s}"] = s

            boundaries_map = self._parse_region_boundaries(sheet_mapping)

            for filename, boundaries in boundaries_map.items():
                if filename not in zf.namelist(): continue
                with zf.open(filename) as f:
                    context = ET.iterparse(f, events=("end",))
                    for event, elem in context:
                        if self._tag_is(elem, "c"):
                            r_attr = elem.get('r')
                            t_attr = elem.get('t')
                            if r_attr:
                                try:
                                    row, col = coordinate_to_tuple(r_attr)
                                    if self._is_in_boundaries(col, row, boundaries):
                                        text_found = None
                                        if t_attr == 's':
                                            v_text = self._get_child_text(elem, "v")
                                            if v_text:
                                                idx = int(v_text)
                                                if 0 <= idx < len(shared_strings):
                                                    text_found = shared_strings[idx]
                                        elif t_attr == 'inlineStr':
                                            is_node = self._find_child(elem, "is")
                                            if is_node is not None:
                                                t_text = self._get_child_text(is_node, "t")
                                                if t_text:
                                                    text_found = t_text
                                        if text_found:
                                            texts_to_translate.add(text_found)
                                except Exception:
                                    pass
                            elem.clear()
                        elif self._tag_is(elem, "row"):
                            elem.clear()
        return list(texts_to_translate)

    def _rebuild_xml_regions(self, original_content_bytes: bytes, translation_map: dict) -> bytes:
        output_zip_io = BytesIO()
        with zipfile.ZipFile(BytesIO(original_content_bytes), 'r') as zf_in:
            with zipfile.ZipFile(output_zip_io, 'w', zipfile.ZIP_DEFLATED) as zf_out:
                shared_strings = self._get_shared_strings(zf_in)
                sheet_mapping = self._get_sheet_mapping(zf_in)
                if not sheet_mapping:
                    all_sheets = [n for n in zf_in.namelist() if
                                  n.startswith("xl/worksheets/sheet") and n.endswith(".xml")]
                    for s in all_sheets:
                        sheet_mapping[f"Unknown_{s}"] = s

                boundaries_map = self._parse_region_boundaries(sheet_mapping)

                for item in zf_in.infolist():
                    if item.filename in boundaries_map:
                        boundaries = boundaries_map[item.filename]
                        with zf_in.open(item.filename) as f:
                            tree = ET.parse(f)
                            root = tree.getroot()
                            cells_modified = False

                            for cell in root.iter():
                                if not self._tag_is(cell, "c"): continue
                                r_attr = cell.get('r')
                                t_attr = cell.get('t')
                                if r_attr:
                                    try:
                                        row, col = coordinate_to_tuple(r_attr)
                                        if self._is_in_boundaries(col, row, boundaries):
                                            original_text = None
                                            if t_attr == 's':
                                                v_text = self._get_child_text(cell, "v")
                                                if v_text:
                                                    idx = int(v_text)
                                                    if 0 <= idx < len(shared_strings):
                                                        original_text = shared_strings[idx]
                                            elif t_attr == 'inlineStr':
                                                is_node = self._find_child(cell, "is")
                                                if is_node is not None:
                                                    original_text = self._get_child_text(is_node, "t")

                                            if original_text and original_text in translation_map:
                                                final_text = self._apply_insert_mode(original_text,
                                                                                     translation_map[original_text])
                                                # 清空旧内容
                                                for child in list(cell):
                                                    cell.remove(child)
                                                # 写入新内容 (inlineStr)
                                                cell.set('t', 'inlineStr')
                                                is_node = ET.Element(f"{{{self.NS_MAIN}}}is")
                                                t_node = ET.SubElement(is_node, f"{{{self.NS_MAIN}}}t")
                                                t_node.text = final_text
                                                if '\n' in final_text or final_text.startswith(
                                                        ' ') or final_text.endswith(' '):
                                                    t_node.set(f"{{{self.NS_XML}}}space", "preserve")
                                                cell.append(is_node)
                                                cells_modified = True
                                    except Exception:
                                        pass
                            if cells_modified:
                                zf_out.writestr(item, ET.tostring(root, encoding='utf-8', xml_declaration=True))
                            else:
                                zf_out.writestr(item, zf_in.read(item.filename))
                    else:
                        zf_out.writestr(item, zf_in.read(item.filename))
        return output_zip_io.getvalue()

    # =========================================================================
    # 全文档处理
    # =========================================================================

    def _get_texts_xml_all(self, document: Document) -> List[str]:
        texts_to_translate = set()
        try:
            with zipfile.ZipFile(BytesIO(document.content), 'r') as zf:
                # 1. Shared Strings
                if "xl/sharedStrings.xml" in zf.namelist():
                    with zf.open("xl/sharedStrings.xml") as f:
                        context = ET.iterparse(f, events=("end",))
                        for event, elem in context:
                            if self._tag_is(elem, "t"):
                                if elem.text and elem.text.strip():
                                    texts_to_translate.add(elem.text)
                                elem.clear()

                # 2. Worksheets
                sheet_files = [n for n in zf.namelist() if n.startswith("xl/worksheets/sheet") and n.endswith(".xml")]
                for sheet_file in sheet_files:
                    with zf.open(sheet_file) as f:
                        context = ET.iterparse(f, events=("end",))
                        for event, elem in context:
                            if self._tag_is(elem, "c"):
                                if elem.get('t') == 'inlineStr':
                                    is_node = self._find_child(elem, "is")
                                    if is_node is not None:
                                        t_text = self._get_child_text(is_node, "t")
                                        if t_text and t_text.strip():
                                            texts_to_translate.add(t_text)
                                elem.clear()
                            elif self._tag_is(elem, "row"):
                                elem.clear()

                # 3. Other Content (Tables / Drawings, 但跳过 Comments)
                for item in zf.infolist():
                    if item.filename.startswith("xl/tables/table"):
                        with zf.open(item.filename) as f:
                            root = ET.fromstring(f.read())
                            for col in root.iter():
                                if self._tag_is(col, "tableColumn") and col.get('name'):
                                    texts_to_translate.add(col.get('name'))

                    # [已移除] 批注 (Comments) 的文本提取逻辑，以跳过翻译

                    # 绘图 (Drawings)
                    elif item.filename.startswith("xl/drawings/drawing") and item.filename.endswith(".xml"):
                        with zf.open(item.filename) as f:
                            context = ET.iterparse(f, events=("end",))
                            for event, elem in context:
                                if self._tag_is(elem, "t") and elem.text and elem.text.strip():
                                    texts_to_translate.add(elem.text)
                                elem.clear()

        except Exception as e:
            self.logger.error(f"XML解析失败: {e}", exc_info=True)
        return list(texts_to_translate)

    def _rebuild_xml_all(self, original_content_bytes: bytes, translation_map: dict) -> bytes:
        output_zip_io = BytesIO()
        try:
            with zipfile.ZipFile(BytesIO(original_content_bytes), 'r') as zf_in:
                with zipfile.ZipFile(output_zip_io, 'w', zipfile.ZIP_DEFLATED) as zf_out:
                    for item in zf_in.infolist():
                        content = zf_in.read(item.filename)
                        filename = item.filename

                        # 1. Shared Strings
                        if filename == "xl/sharedStrings.xml":
                            root = ET.fromstring(content)
                            modified = False
                            for si in root.iter():
                                if self._tag_is(si, "si"):
                                    # 处理直接的 t
                                    direct_t = self._find_child(si, "t")
                                    if direct_t is not None and direct_t.text in translation_map:
                                        new_text = self._apply_insert_mode(direct_t.text,
                                                                           translation_map[direct_t.text])
                                        direct_t.text = new_text
                                        if '\n' in new_text or new_text.strip() != new_text:
                                            direct_t.set(f"{{{self.NS_XML}}}space", "preserve")
                                        # 移除拼音
                                        p_pr = self._find_child(si, "phoneticPr")
                                        if p_pr is not None: si.remove(p_pr)
                                        modified = True

                                    # 处理 Rich Text runs (<r>)
                                    for r in si.iter():
                                        if self._tag_is(r, "r"):
                                            t_node = self._find_child(r, "t")
                                            if t_node is not None and t_node.text in translation_map:
                                                new_text = self._apply_insert_mode(t_node.text,
                                                                                   translation_map[t_node.text])
                                                t_node.text = new_text
                                                if '\n' in new_text or new_text.strip() != new_text:
                                                    t_node.set(f"{{{self.NS_XML}}}space", "preserve")
                                                modified = True

                            if modified:
                                zf_out.writestr(item, ET.tostring(root, encoding='utf-8', xml_declaration=True))
                            else:
                                zf_out.writestr(item, content)

                        # 2. Worksheets
                        elif filename.startswith("xl/worksheets/sheet") and filename.endswith(".xml"):
                            root = ET.fromstring(content)
                            modified = False
                            for cell in root.iter():
                                if self._tag_is(cell, "c") and cell.get('t') == 'inlineStr':
                                    is_node = self._find_child(cell, "is")
                                    if is_node is not None:
                                        t_node = self._find_child(is_node, "t")
                                        if t_node is not None and t_node.text in translation_map:
                                            new_text = self._apply_insert_mode(t_node.text,
                                                                               translation_map[t_node.text])
                                            t_node.text = new_text
                                            if '\n' in new_text or new_text.strip() != new_text:
                                                t_node.set(f"{{{self.NS_XML}}}space", "preserve")
                                            modified = True
                            if modified:
                                zf_out.writestr(item, ET.tostring(root, encoding='utf-8', xml_declaration=True))
                            else:
                                zf_out.writestr(item, content)

                        # 3. Tables
                        elif filename.startswith("xl/tables/table"):
                            root = ET.fromstring(content)
                            modified = False
                            for col in root.iter():
                                if self._tag_is(col, "tableColumn"):
                                    orig = col.get('name')
                                    if orig in translation_map:
                                        col.set('name', self._apply_insert_mode(orig, translation_map[orig]))
                                        modified = True
                            if modified:
                                zf_out.writestr(item, ET.tostring(root, encoding='utf-8', xml_declaration=True))
                            else:
                                zf_out.writestr(item, content)

                        # [已移除] 批注 (Comments) 的重构逻辑
                        # 4. Comments: 直接跳过处理，原样写入，防止报错

                        # 5. Drawings (保留处理)
                        elif filename.startswith("xl/drawings/drawing") and filename.endswith(".xml"):
                            root = ET.fromstring(content)
                            modified = False
                            for elem in root.iter():
                                if self._tag_is(elem, "p"):  # 匹配 a:p
                                    for child in list(elem):  # 转为 list 以便支持 remove
                                        if self._tag_is(child, "r"):  # a:r
                                            t_node = self._find_child(child, "t")
                                            if t_node is not None and t_node.text in translation_map:
                                                new_text = self._apply_insert_mode(t_node.text,
                                                                                   translation_map[t_node.text])
                                                t_node.text = new_text
                                                if '\n' in new_text or new_text.strip() != new_text:
                                                    t_node.set(f"{{{self.NS_XML}}}space", "preserve")
                                                modified = True
                            if modified:
                                zf_out.writestr(item, ET.tostring(root, encoding='utf-8', xml_declaration=True))
                            else:
                                zf_out.writestr(item, content)

                        else:
                            zf_out.writestr(item, content)
            return output_zip_io.getvalue()
        except Exception as e:
            self.logger.error(f"XML重构失败: {e}", exc_info=True)
            return original_content_bytes

    def translate(self, document: Document) -> Self:
        if self.translate_regions:
            original_texts = self._get_texts_xml_regions(document)
        else:
            original_texts = self._get_texts_xml_all(document)

        if not original_texts:
            print(f"\n未找到需要翻译的文本 (模式: {'区域' if self.translate_regions else '全文档'}).")
            return self

        if self.glossary_agent:
            glossary_dict_gen = self.glossary_agent.send_segments(original_texts, self.chunk_size)
            if self.glossary:
                self.glossary.update(glossary_dict_gen)
            if self.translate_agent and self.glossary:
                self.translate_agent.update_glossary_dict(self.glossary.glossary_dict)

        if self.translate_agent:
            translated_texts = self.translate_agent.send_segments(original_texts, self.chunk_size)
        else:
            translated_texts = original_texts

        translation_map = dict(zip(original_texts, translated_texts))

        if self.translate_regions:
            document.content = self._rebuild_xml_regions(document.content, translation_map)
        else:
            document.content = self._rebuild_xml_all(document.content, translation_map)

        return self

    async def translate_async(self, document: Document) -> Self:
        if self.translate_regions:
            original_texts = await asyncio.to_thread(self._get_texts_xml_regions, document)
        else:
            original_texts = await asyncio.to_thread(self._get_texts_xml_all, document)

        if not original_texts:
            print(f"\n未找到需要翻译的文本 (模式: {'区域' if self.translate_regions else '全文档'}).")
            return self

        if self.glossary_agent:
            glossary_dict_gen = await self.glossary_agent.send_segments_async(original_texts, self.chunk_size)
            if self.glossary:
                self.glossary.update(glossary_dict_gen)
            if self.translate_agent and self.glossary:
                self.translate_agent.update_glossary_dict(self.glossary.glossary_dict)

        if self.translate_agent:
            translated_texts = await self.translate_agent.send_segments_async(original_texts, self.chunk_size)
        else:
            translated_texts = original_texts

        translation_map = dict(zip(original_texts, translated_texts))

        if self.translate_regions:
            document.content = await asyncio.to_thread(self._rebuild_xml_regions, document.content, translation_map)
        else:
            document.content = await asyncio.to_thread(self._rebuild_xml_all, document.content, translation_map)

        return self


# SPDX-FileCopyrightText: 2025 QinHan
# SPDX-License-Identifier: MPL-2.0
"""
DocuTranslate MCP Server Package

This package provides a Model Context Protocol (MCP) server for DocuTranslate,
enabling AI assistants to use document translation capabilities.

Features:
- Async task queue mode: submit translation and get task_id immediately
- Task status querying
- Multiple transport protocols support (stdio, sse, streamable-http)
- Can be mounted to existing FastAPI app

Quick Start:
    ```bash
    # Install with MCP dependencies
    pip install docutranslate[mcp]

    # Run stdio server (for Claude Desktop, etc.)
    docutranslate --mcp

    # Run SSE server (for Cherry Studio)
    docutranslate --mcp --transport sse
    ```

Mount to existing FastAPI:
    ```python
    from fastapi import FastAPI
    from docutranslate.mcp import get_sse_app

    app = FastAPI()
    mcp_app = get_sse_app()
    app.mount("/mcp", mcp_app)
    ```

For more details, see the README.md in this package.
"""

try:
    from docutranslate.mcp.server import (
        create_mcp_server,
        run_mcp_server,
        get_sse_app,
        MCP_AVAILABLE,
    )
except ImportError:
    # MCP dependencies not installed
    MCP_AVAILABLE = False

    def _mcp_not_available(*args, **kwargs):
        raise ImportError(
            "MCP dependencies not installed. "
            "Install with: pip install docutranslate[mcp]"
        )

    create_mcp_server = _mcp_not_available
    run_mcp_server = _mcp_not_available
    get_sse_app = _mcp_not_available

__all__ = [
    "create_mcp_server",
    "run_mcp_server",
    "get_sse_app",
    "MCP_AVAILABLE",
]



# SPDX-FileCopyrightText: 2025 QinHan
# SPDX-License-Identifier: MPL-2.0
import asyncio
from collections import defaultdict
from copy import deepcopy
from dataclasses import dataclass
from io import BytesIO
from typing import Self, Literal, List, Dict, Any, Tuple

import docx
from docx.document import Document as DocumentObject
from docx.opc.part import Part
from docx.oxml import OxmlElement
from docx.oxml.ns import qn
from docx.oxml.text.run import CT_R
from docx.section import _Header, _Footer
from docx.text.paragraph import Paragraph
from docx.text.run import Run
from docx.table import _Cell, Table

from docutranslate.agents.segments_agent import SegmentsTranslateAgentConfig, SegmentsTranslateAgent
from docutranslate.ir.document import Document
from docutranslate.translator.ai_translator.base import AiTranslatorConfig, AiTranslator

# ---------------- 辅助函数 ----------------

SIGNIFICANT_STYLES = frozenset([
    qn('w:u'),  # 下划线
    qn('w:strike'),  # 删除线
    qn('w:dstrike'),  # 双删除线
    qn('w:shd'),  # 底纹/背景色
    qn('w:highlight'),  # 荧光笔高亮
    qn('w:bdr'),  # 边框
    qn('w:effectLst'),  # 文本效果 (如发光、阴影)
    qn('w:em'),  # 强调标记 (着重号)
])


def is_image_run(run: Run) -> bool:
    """检查一个 Run 是否包含图片。"""
    xml = getattr(run.element, 'xml', '')
    return '<w:drawing' in xml or '<w:pict' in xml


def is_formatting_only_run(run: Run) -> bool:
    """
    检查一个 Run 是否仅用于格式化，不包含任何应被渲染的文本。
    这仅适用于其 .text 属性为 "" 的情况。
    """
    return run.text == ""


def is_tab_run(run: Run) -> bool:
    """
    检查一个 Run 是否主要代表一个制表符，应被视作格式边界。
    """
    if run.text.strip():
        return False
    xml = getattr(run.element, 'xml', '')
    return '<w:tab' in xml or '<w:ptab' in xml


def is_instr_text_run(run: Run) -> bool:
    """
    检查一个 Run 是否包含域指令文本 (w:instrText)。
    目录(TOC)、页码、超链接等功能的指令代码存储在此标签中。
    必须跳过这些 Run，否则写入 text 会破坏域结构。
    """
    return run.element.find(qn('w:instrText')) is not None


# ---------------- 配置类 ----------------
@dataclass
class DocxTranslatorConfig(AiTranslatorConfig):
    insert_mode: Literal["replace", "append", "prepend"] = "replace"
    separator: str = "\n"


# ---------------- 主类 ----------------
class DocxTranslator(AiTranslator):
    """
    一个基于高级结构化解析的 .docx 文件翻译器。
    它能高精度保留样式，并正确处理正文、表格、页眉/脚、脚注/尾注、超链接和目录(TOC)等复杂元素。
    """
    IGNORED_TAGS = {
        qn('w:proofErr'), qn('w:lastRenderedPageBreak'), qn('w:bookmarkStart'),
        qn('w:bookmarkEnd'), qn('w:commentRangeStart'), qn('w:commentRangeEnd'),
        qn('w:del'), qn('w:ins'), qn('w:moveFrom'), qn('w:moveTo'),
    }
    RECURSIVE_CONTAINER_TAGS = {
        qn('w:smartTag'), qn('w:sdtContent'), qn('w:hyperlink'),
    }

    def __init__(self, config: DocxTranslatorConfig):
        super().__init__(config=config)
        self.chunk_size = config.chunk_size
        self.translate_agent = None
        glossary_dict = self.glossary.glossary_dict if self.glossary else None
        if not self.skip_translate:
            agent_config = SegmentsTranslateAgentConfig(
                custom_prompt=config.custom_prompt, to_lang=config.to_lang, base_url=config.base_url,
                api_key=config.api_key, model_id=config.model_id, temperature=config.temperature,
                thinking=config.thinking, concurrent=config.concurrent, timeout=config.timeout,
                logger=self.logger, glossary_dict=glossary_dict, retry=config.retry,
                system_proxy_enable=config.system_proxy_enable, force_json=config.force_json,
                rpm=config.rpm,
                tpm=config.tpm,
                provider=config.provider,
                extra_body=config.extra_body,
            )
            self.translate_agent = SegmentsTranslateAgent(agent_config)
        self.insert_mode = config.insert_mode
        self.separator = config.separator

    def _get_significant_styles(self, run: Run) -> frozenset:
        """从一个 Run 中提取“显著”格式标签的集合。"""
        if run is None:
            return frozenset()
        rPr = run.element.rPr
        if rPr is None:
            return frozenset()
        return frozenset(child.tag for child in rPr if child.tag in SIGNIFICANT_STYLES)

    def _have_same_significant_styles(self, run1: Run, run2: Run) -> bool:
        """检查两个 Run 是否具有相同的“显著”格式集合。"""
        styles1 = self._get_significant_styles(run1)
        styles2 = self._get_significant_styles(run2)
        return styles1 == styles2

    def _process_element_children(self, element, parent_paragraph: Paragraph, elements: List[Dict[str, Any]],
                                  texts: List[str],
                                  state: Dict[str, Any],
                                  top_level_para: Paragraph):

        def flush_segment():
            current_runs = state['current_runs']
            if not current_runs:
                return
            full_text = "".join(r.text for r in current_runs)
            if full_text.strip():
                # 在 elements 中增加对父段落和顶级段落的引用
                elements.append({
                    "type": "text_runs",
                    "runs": list(current_runs),
                    "paragraph": parent_paragraph,
                    "top_level_paragraph": top_level_para
                })
                texts.append(full_text)
            state['current_runs'].clear()

        for child in element:
            if child.tag in self.IGNORED_TAGS:
                continue

            if child.tag in self.RECURSIVE_CONTAINER_TAGS:
                flush_segment()
                self._process_element_children(child, parent_paragraph, elements, texts, state, top_level_para)
                flush_segment()  # 在递归容器后也刷新，确保其内容成为独立片段
                continue

            field_char_element = child.find(qn('w:fldChar')) if isinstance(child, CT_R) else None
            if field_char_element is not None:
                fld_type = field_char_element.get(qn('w:fldCharType'))
                if fld_type == 'begin' or fld_type == 'end':
                    flush_segment()
                continue

            if isinstance(child, CT_R):
                # 传入 parent_paragraph 以确保 Run 对象具有正确的上下文
                run = Run(child, parent_paragraph)

                if '<w:drawing' in run.element.xml or '<w:pict' in run.element.xml:
                    text_boxes = list(run.element.iter(qn('w:txbxContent')))
                    if text_boxes:
                        flush_segment()  # 包含文本的形状是一个边界，刷新前面的文本
                        for txbx_content in text_boxes:
                            for p_element in txbx_content.findall(qn('w:p')):
                                shape_para = Paragraph(p_element, parent_paragraph)
                                self._process_paragraph(shape_para, elements, texts, top_level_para=top_level_para)
                        continue

                # 增加对 instrText 的检查，防止提取出 TOC 指令作为原文
                if is_image_run(run) or is_formatting_only_run(run) or is_tab_run(run) or is_instr_text_run(run):
                    flush_segment()
                    continue

                # 保留原有逻辑: 基于格式变化进行切分
                last_run_in_segment = state['current_runs'][-1] if state['current_runs'] else None
                if last_run_in_segment and not self._have_same_significant_styles(last_run_in_segment, run):
                    flush_segment()

                state['current_runs'].append(run)
            else:
                flush_segment()

    def _process_paragraph(self, para: Paragraph, elements: List[Dict[str, Any]], texts: List[str],
                           top_level_para: Paragraph = None):
        if top_level_para is None:
            top_level_para = para

        state = {
            'current_runs': [],
        }
        self._process_element_children(para._p, para, elements, texts, state, top_level_para)

        current_runs = state['current_runs']
        if current_runs:
            full_text = "".join(r.text for r in current_runs)
            if full_text.strip():
                elements.append({
                    "type": "text_runs",
                    "runs": list(current_runs),
                    "paragraph": para,
                    "top_level_paragraph": top_level_para
                })
                texts.append(full_text)
            current_runs.clear()

    def _process_body_elements(self, parent_element, container, elements: List[Dict[str, Any]], texts: List[str]):
        """ 遍历一个容器内的所有顶级元素（段落、表格、内容控件等） """
        for child_element in parent_element:
            if child_element.tag.endswith('p'):
                self._process_paragraph(Paragraph(child_element, container), elements, texts)
            elif child_element.tag.endswith('tbl'):
                table = Table(child_element, container)
                for row in table.rows:
                    for cell in row.cells:
                        self._traverse_container(cell, elements, texts)
            elif child_element.tag.endswith('sdt'):
                sdt_content = child_element.find(qn('w:sdtContent'))
                if sdt_content is not None:
                    self._process_body_elements(sdt_content, container, elements, texts)

    def _traverse_container(self, container: Any, elements: List[Dict[str, Any]], texts: List[str]):
        if container is None:
            return

        parent_element = None
        if isinstance(container, (DocumentObject, Part)):
            parent_element = container.element.body if hasattr(container.element, 'body') else container.element
        elif isinstance(container, (_Cell, _Header, _Footer)):
            parent_element = container._element
        else:
            self.logger.warning(f"跳过未知类型的容器: {type(container)}")
            return

        if parent_element is not None and parent_element.tag in [qn('w:footnotes'), qn('w:endnotes')]:
            for note_element in parent_element:
                self._process_body_elements(note_element, container, elements, texts)
        elif parent_element is not None:
            self._process_body_elements(parent_element, container, elements, texts)

    def _pre_translate(self, document: Document) -> Tuple[DocumentObject, List[Dict[str, Any]], List[str]]:
        doc = docx.Document(BytesIO(document.content))
        elements, texts = [], []

        self._traverse_container(doc, elements, texts)

        for section in doc.sections:
            self._traverse_container(section.header, elements, texts)
            self._traverse_container(section.first_page_header, elements, texts)
            self._traverse_container(section.even_page_header, elements, texts)
            self._traverse_container(section.footer, elements, texts)
            self._traverse_container(section.first_page_footer, elements, texts)
            self._traverse_container(section.even_page_footer, elements, texts)

        if hasattr(doc.part, 'footnotes_part') and doc.part.footnotes_part is not None:
            self._traverse_container(doc.part.footnotes_part, elements, texts)
        if hasattr(doc.part, 'endnotes_part') and doc.part.endnotes_part is not None:
            self._traverse_container(doc.part.endnotes_part, elements, texts)

        return doc, elements, texts

    def _apply_translation(self, element_info: Dict[str, Any], final_text: str):
        if element_info["type"] == "text_runs":
            runs = element_info["runs"]
            if not runs: return

            first_real_run_index = -1
            for i, run in enumerate(runs):
                if run.element.getparent() is not None:
                    run._parent = element_info["paragraph"]
                    run.text = final_text
                    first_real_run_index = i
                    break

            if first_real_run_index == -1:
                self.logger.warning(f"无法应用翻译 '{final_text}'，因为找不到有效的run。")
                return

            for i in range(first_real_run_index + 1, len(runs)):
                run = runs[i]
                parent_element = run.element.getparent()
                if parent_element is not None:
                    try:
                        parent_element.remove(run.element)
                    except ValueError:
                        self.logger.debug(f"尝试删除一个不存在的run元素。这通常是安全的。")
                        pass

    def _prune_unwanted_elements_from_copy(self, p_element: OxmlElement):
        """
        从复制的段落元素中移除图片、页码字段以及TOC域指令。
        这可以防止在“append”模式下出现重复的图片、错误的页码或裸露的域代码。
        """
        runs_to_remove = []
        runs = p_element.findall(qn('w:r'))

        i = 0
        while i < len(runs):
            run_element = runs[i]

            # 1. 检查图片
            if run_element.find(qn('w:drawing')) is not None or run_element.find(qn('w:pict')) is not None:
                runs_to_remove.append(run_element)
                i += 1
                continue

            # 2. 检查域开始字符 (处理 PAGE, NUMPAGES, TOC 等)
            fldChar = run_element.find(qn('w:fldChar'))
            if fldChar is not None and fldChar.get(qn('w:fldCharType')) == 'begin':
                is_target_field = False
                field_end_index = -1
                is_toc_field = False

                # 向前查找指令文本
                for j in range(i + 1, len(runs)):
                    next_run = runs[j]

                    # 检查指令文本
                    instrText = next_run.find(qn('w:instrText'))
                    if instrText is not None and instrText.text is not None:
                        text = instrText.text.strip().upper()
                        # 检测页码
                        if 'PAGE' in text or 'NUMPAGES' in text:
                            is_target_field = True
                            break
                        # [FIX] 检测 TOC 目录生成指令
                        # 如果是 TOC 指令，我们必须移除它，否则译文段落会再次尝试生成目录，或显示乱码。
                        if text.startswith('TOC'):
                            is_target_field = True
                            is_toc_field = True
                            break

                    # 如果遇到嵌套的 begin，或者 unexpected end，停止搜索
                    next_fldChar = next_run.find(qn('w:fldChar'))
                    if next_fldChar is not None:
                        if next_fldChar.get(qn('w:fldCharType')) == 'begin':
                            break  # 嵌套开始，放弃当前层匹配（简化处理）
                        if next_fldChar.get(qn('w:fldCharType')) == 'end':
                            # 到了结束还没找到指令，说明不是我们要找的字段
                            break

                if is_target_field:
                    # 找到要移除的字段了。
                    # 对于 PAGE 字段，我们通常移除整个字段（因为译文段落的页码可能不准确，或者避免重复）。
                    # 对于 TOC 字段，我们必须"解包"：移除 Begin, Instr, Separate，但保留结果文本(Hyperlink/Text)。
                    # 但在这里，简单的策略是：移除 Begin 到 Separate (或 End) 之间的指令部分。

                    if is_toc_field:
                        # 特殊处理 TOC：我们希望保留后面的文字（如 "研究背景..."），只删掉域的定义部分。
                        # TOC 结构通常是: Begin -> Instr(TOC) -> Separate -> Result(Text) -> End
                        # 我们移除 Begin -> ... -> Separate。保留 Result -> End。
                        runs_to_remove.append(run_element)  # 移除 Begin

                        found_separate = False
                        for j in range(i + 1, len(runs)):
                            field_run = runs[j]

                            # 总是移除中间的 run (包含 instrText)
                            runs_to_remove.append(field_run)

                            end_fldChar = field_run.find(qn('w:fldChar'))
                            if end_fldChar is not None:
                                fld_type = end_fldChar.get(qn('w:fldCharType'))
                                if fld_type == 'separate':
                                    found_separate = True
                                    i = j + 1  # 跳过 Separate，继续处理后面的 Result Run
                                    break
                                if fld_type == 'end':
                                    # 如果没有 separate 直接 end，那就全删了
                                    i = j + 1
                                    break
                        if found_separate:
                            continue  # 继续外层循环

                    else:
                        # 普通字段 (PAGE 等)，移除整个字段 (Begin ... End)
                        field_runs_to_remove = [run_element]
                        end_found = False
                        for j in range(i + 1, len(runs)):
                            field_run = runs[j]
                            field_runs_to_remove.append(field_run)
                            end_fldChar = field_run.find(qn('w:fldChar'))
                            if end_fldChar is not None and end_fldChar.get(qn('w:fldCharType')) == 'end':
                                end_found = True
                                field_end_index = j
                                break

                        if end_found:
                            runs_to_remove.extend(field_runs_to_remove)
                            i = field_end_index + 1
                            continue

            i += 1

        # 从 XML 树中实际移除被标记的 runs
        for run_to_remove in runs_to_remove:
            if run_to_remove.getparent() is not None:
                p_element.remove(run_to_remove)

    def _after_translate(self, doc: DocumentObject, elements: List[Dict[str, Any]], translated: List[str],
                         originals: List[str]) -> bytes:
        if len(elements) != len(translated):
            self.logger.error(
                f"翻译数量不匹配！原文: {len(originals)}, 译文: {len(translated)}. 将只处理公共部分。")
            min_len = min(len(elements), len(translated), len(originals))
            elements, translated, originals = elements[:min_len], translated[:min_len], originals[:min_len]

        if self.insert_mode == "replace":
            for info, trans in zip(elements, translated):
                self._apply_translation(info, trans)
        else:
            paragraph_segments = defaultdict(list)
            for i, info in enumerate(elements):
                top_level_paragraph = info["top_level_paragraph"]
                paragraph_segments[id(top_level_paragraph._p)].append({
                    "index": i,
                    "translation": translated[i],
                    "paragraph_obj": top_level_paragraph
                })

            for para_id, segments_for_this_para in paragraph_segments.items():
                top_level_paragraph_orig = segments_for_this_para[0]["paragraph_obj"]
                p_element_orig = top_level_paragraph_orig._p

                translated_p_element = deepcopy(p_element_orig)

                # ================= 核心修复：先映射 =================
                top_level_paragraph_copy = Paragraph(translated_p_element, top_level_paragraph_orig._parent)
                para_map = {id(p_element_orig): top_level_paragraph_copy}
                orig_nested_ps = p_element_orig.iter(qn('w:p'))
                copy_nested_ps = translated_p_element.iter(qn('w:p'))
                for o, c in zip(orig_nested_ps, copy_nested_ps):
                    para_map[id(o)] = Paragraph(c, top_level_paragraph_copy)

                # 建立 Run 映射
                run_element_map = {
                    id(orig_r): copied_r
                    for orig_r, copied_r in zip(p_element_orig.iter(qn('w:r')), translated_p_element.iter(qn('w:r')))
                }

                # ================= 核心修复：后清理 =================
                # 移除 TOC 指令等干扰元素，确保译文纯净
                self._prune_unwanted_elements_from_copy(translated_p_element)

                # ================= 新增逻辑：判断是否需要使用软回车 =================
                use_soft_break = False
                pPr = p_element_orig.find(qn('w:pPr'))

                if pPr is not None:
                    # 1. 判断是否列表项
                    if pPr.find(qn('w:numPr')) is not None:
                        use_soft_break = True

                    # 2. 判断是否目录项 (样式通常为 TOC1, TOC2...)
                    pStyle = pPr.find(qn('w:pStyle'))
                    if pStyle is not None:
                        style_val = pStyle.get(qn('w:val'), "")
                        # 检查样式名称是否以 TOC 开头 (忽略大小写)
                        if style_val and style_val.upper().startswith("TOC"):
                            use_soft_break = True

                # 如果不是软回车模式，才需要在副本中清理列表属性（避免两个圆点）
                if not use_soft_break:
                    pPr_copy = translated_p_element.find(qn('w:pPr'))
                    if pPr_copy is not None:
                        numPr = pPr_copy.find(qn('w:numPr'))
                        if numPr is not None:
                            pPr_copy.remove(numPr)

                # 应用翻译到副本 (使用前面建立的 map)
                for seg_info in segments_for_this_para:
                    element_index = seg_info["index"]
                    translation = seg_info["translation"]
                    original_element_info = elements[element_index]

                    original_para_id = id(original_element_info["paragraph"]._p)
                    translated_paragraph_obj = para_map.get(original_para_id)

                    if not translated_paragraph_obj: continue

                    runs_from_copy = []
                    for r in original_element_info["runs"]:
                        copied_r_element = run_element_map.get(id(r.element))
                        # 只有当该 Run 在副本中未被 prune 删除时，才进行翻译替换
                        if copied_r_element is not None and copied_r_element.getparent() is not None:
                            new_run = Run(copied_r_element, translated_paragraph_obj)
                            runs_from_copy.append(new_run)

                    if runs_from_copy:
                        self._apply_translation({
                            "type": "text_runs", "runs": runs_from_copy, "paragraph": translated_paragraph_obj
                        }, translation)

                # ================= 插入逻辑分支 =================
                if use_soft_break:
                    # 列表项或目录项：使用软回车 (<w:br>) 连接，忽略 separator
                    separator_run = OxmlElement('w:r')
                    separator_run.append(OxmlElement('w:br'))

                    translated_runs = list(translated_p_element.iter(qn('w:r')))

                    if self.insert_mode == "append":
                        p_element_orig.append(separator_run)
                        for tr in translated_runs:
                            p_element_orig.append(tr)

                    elif self.insert_mode == "prepend":
                        # 如果有属性节点(pPr)，插入到它后面
                        insert_index = 0
                        if p_element_orig.find(qn('w:pPr')) is not None:
                            insert_index = 1

                        # 先插入分隔符
                        p_element_orig.insert(insert_index, separator_run)
                        # 倒序插入译文 runs，确保顺序正确
                        for tr in reversed(translated_runs):
                            p_element_orig.insert(insert_index, tr)

                else:
                    # 普通段落：使用新段落 (<w:p>) 插入，并使用 separator
                    separator_p_element = None
                    if self.separator:
                        separator_p_element = OxmlElement('w:p')
                        # 尝试复制原段落样式到分隔符段落，保持间距一致（可选）
                        if pPr is not None:
                            separator_p_element.append(deepcopy(pPr))
                            # 但要去掉列表属性和边框等，防止分隔符也带列表头
                            sep_pPr = separator_p_element.find(qn('w:pPr'))
                            if sep_pPr is not None:
                                numPr = sep_pPr.find(qn('w:numPr'))
                                if numPr is not None: sep_pPr.remove(numPr)

                        run_element = OxmlElement('w:r')
                        lines = self.separator.split('\n')
                        for i, line in enumerate(lines):
                            text_element = OxmlElement('w:t')
                            text_element.set(qn('xml:space'), 'preserve')
                            text_element.text = line
                            run_element.append(text_element)
                            if i < len(lines) - 1:
                                run_element.append(OxmlElement('w:br'))
                        separator_p_element.append(run_element)

                    if self.insert_mode == "append":
                        current_element = p_element_orig
                        if separator_p_element is not None:
                            current_element.addnext(separator_p_element)
                            current_element = separator_p_element
                        current_element.addnext(translated_p_element)
                    elif self.insert_mode == "prepend":
                        p_element_orig.addprevious(translated_p_element)
                        if separator_p_element is not None:
                            translated_p_element.addnext(separator_p_element)

        doc_output_stream = BytesIO()
        doc.save(doc_output_stream)
        return doc_output_stream.getvalue()

    def translate(self, document: Document) -> Self:
        doc, elements, originals = self._pre_translate(document)
        if not originals:
            self.logger.info("\n文档中未找到可翻译的文本内容。")
            document.content = self._after_translate(doc, elements, [], [])
            return self

        if self.glossary_agent:
            glossary_dict_gen = self.glossary_agent.send_segments(originals, self.chunk_size)
            if self.glossary:
                self.glossary.update(glossary_dict_gen)
            if self.translate_agent and self.glossary:
                self.translate_agent.update_glossary_dict(self.glossary.glossary_dict)

        translated = self.translate_agent.send_segments(originals,
                                                        self.chunk_size) if self.translate_agent else originals
        document.content = self._after_translate(doc, elements, translated, originals)
        return self

    async def translate_async(self, document: Document) -> Self:
        doc, elements, originals = await asyncio.to_thread(self._pre_translate, document)
        if not originals:
            self.logger.info("\n文档中未找到可翻译的文本内容。")
            document.content = await asyncio.to_thread(self._after_translate, doc, elements, [], [])
            return self

        if self.glossary_agent:
            glossary_dict_gen = await self.glossary_agent.send_segments_async(originals, self.chunk_size)
            if self.glossary:
                self.glossary.update(glossary_dict_gen)
            if self.translate_agent and self.glossary:
                self.translate_agent.update_glossary_dict(self.glossary.glossary_dict)

        translated = await self.translate_agent.send_segments_async(originals,
                                                                    self.chunk_size) if self.translate_agent else originals
        document.content = await asyncio.to_thread(self._after_translate, doc, elements, translated, originals)
        return self


# SPDX-FileCopyrightText: 2025 QinHan
# SPDX-License-Identifier: MPL-2.0

import asyncio
import json
import re
from dataclasses import dataclass
from json import JSONDecodeError
from logging import Logger

from json_repair import json_repair

from docutranslate.agents import AgentConfig, Agent
from docutranslate.agents.agent import PartialAgentResultError, AgentResultError
from docutranslate.glossary.glossary import Glossary
from docutranslate.utils.json_utils import segments2json_chunks, fix_json_string, parse_json_response


def generate_prompt(json_segments: str, to_lang: str):
    return f"""
You will receive a sequence of original text segments to be translated, represented in JSON format. The keys are segment IDs, and the values are the text content to be translated.    
Here is the input:

<input>
```json
{json_segments}
```
</input>

For each Key-Value Pair in the JSON, translate the contents of the value into {to_lang}, Write the translation back into the value for that JSON.
> (Very important) The original text segments and translated segments must strictly correspond one-to-one. It is strictly forbidden for the IDs of the translated segments to differ from those of the original segments.
> The segment IDs in the output must exactly match those in the input. And all segment IDs in input must appear in the output.
> If necessary, two segments can only be translated together, the translation should be proportionally allocated to the corresponding key's value based on the word count ratio of the segments.

Here is an example of the expected format:

<example>
Input:

```json
{{"3":"source text 3","4":"source text 4"}}
```

Output(target language: {to_lang}):

```json
[{{"id":"3","t":"translated 3"}},{{"id":"4","t":"translated 4"}}]
```

Note: Use "id" for the segment ID and "t" for the translated text.
For statements that must be combined during translation, employ merging at the minimal structural level. The total number of keys must remain unchanged after merging, and any empty values should be retained.
Below is an example of how merging should be done when necessary:

input:
```json
{{"3":"汤姆说:“杰克你","4":"好”。"}}
```
output:
```json
[{{"id":"3","t":"Tom says:\"Hello Jack.\""}},{{"id":"4","t":""}}]
```
</example>
Please return the translated JSON directly without including any additional information and preserve special tags or untranslatable elements (such as code, brand names, technical terms) as they are.
"""


def get_original_segments(prompt: str):
    match = re.search(r'<input>\n```json\n(.*)\n```\n</input>', prompt, re.DOTALL)
    if match:
        return match.group(1)
    else:
        raise ValueError("无法从prompt中提取初始文本")


def get_target_segments(result: str):
    """使用统一解析函数解析JSON响应"""
    return parse_json_response(result)


@dataclass(kw_only=True)
class SegmentsTranslateAgentConfig(AgentConfig):
    to_lang: str
    custom_prompt: str | None = None
    glossary_dict: dict[str, str] | None = None
    force_json:bool = False


class SegmentsTranslateAgent(Agent):
    def __init__(self, config: SegmentsTranslateAgentConfig):
        super().__init__(config)
        self.to_lang = config.to_lang
        self.force_json = config.force_json
        self.system_prompt = f"""
# Role
- You are a professional, authentic machine translation engine.
"""
        self.custom_prompt = config.custom_prompt
        if config.custom_prompt:
            self.system_prompt += "\n# **Important rules or background** \n" + self.custom_prompt + '\nEND\n'
        self.glossary_dict = config.glossary_dict

    def _pre_send_handler(self, system_prompt, prompt):
        if self.glossary_dict:
            glossary = Glossary(glossary_dict=self.glossary_dict)
            system_prompt += glossary.append_system_prompt(prompt)
        return system_prompt, prompt

    def get_continue_prompt(self, accumulated_result: str, prompt: str) -> str:
        """
        继续获取时的提示词。
        只返回新增的数组元素，而不是完整的JSON数组。
        """
        # 从原始prompt中提取原文的ID范围，帮助模型正确继续
        try:
            original_segments = get_original_segments(prompt)
            original_data = json_repair.loads(original_segments)
            original_ids = list(original_data.keys())
            if original_ids:
                min_id = min(original_ids, key=lambda x: int(x) if x.isdigit() else float('inf'))
                max_id = max(original_ids, key=lambda x: int(x) if x.isdigit() else 0)
                id_range_info = f"原文ID范围: {min_id} - {max_id}"
            else:
                id_range_info = ""
        except Exception:
            id_range_info = ""

        return f"""你之前的翻译输出被截断了。

之前已输出的内容:
```json
{accumulated_result}
```

请继续输出后续的翻译内容。只输出新增的数组元素，格式为JSON数组。

{id_range_info}

重要规则：
- 仔细分析之前已输出的内容，找出最后一个已翻译的ID
- 只输出从"最后一个已翻译的ID + 1"开始的新ID
- 不要重复之前已输出的任何元素
- 例如：如果之前最后输出到 id="3"，请继续从 id="4" 开始
- 只输出新增的部分，不要输出之前已有的内容
- 格式：[{{"id":"4","t":"翻译4"}}, {{"id":"5","t":"翻译5"}}]
"""

    def merge_continue_result(self, accumulated_result: str, additional_result: str) -> str:
        """
        合并继续获取的结果。
        处理追加模式的数组合并：将追加的数组元素合并到已有的数组中。
        自动去重：如果 additional 中有重复的 ID，保留 accumulated 中的版本。
        """
        try:
            # 尝试解析两个部分
            accumulated = parse_json_response(accumulated_result)
            additional = parse_json_response(additional_result)

            # 如果都是列表，合并并去重
            if isinstance(accumulated, list) and isinstance(additional, list):
                # 收集 accumulated 中的 ID
                existing_ids = {item.get("id") for item in accumulated if "id" in item}

                # 只添加 additional 中不重复的 ID
                for item in additional:
                    if item.get("id") not in existing_ids:
                        accumulated.append(item)
                        existing_ids.add(item.get("id"))

                return json.dumps(accumulated, ensure_ascii=False)
        except Exception as e:
            # 如果解析失败，回退到直接拼接
            pass
        return accumulated_result + additional_result

    def _result_handler(self, result: str, origin_prompt: str, logger: Logger):
        """
        处理成功的API响应。
        - 输入格式: [{"id":"1","t":"翻译"}, ...]
        - 输出格式: {"1": "翻译", "2": "翻译2"}
        - 如果ID完全匹配，返回翻译结果。
        - 如果ID不匹配，构造一个部分成功的结果，并通过 PartialTranslationError 异常抛出，以触发重试。
        - 其他错误（如JSON解析失败、模型偷懒）则抛出普通 ValueError 触发重试。
        """
        original_segments = get_original_segments(origin_prompt)
        repaired_result = get_target_segments(result)  # 直接返回解析后的 list 或 dict
        if not repaired_result:
            if original_segments.strip() != "":
                raise AgentResultError("result为空值但原文不为空")
            return {}

        try:
            original_chunk = json_repair.loads(original_segments)

            # 兼容处理：如果返回的是 dict（旧格式）而非 array
            if isinstance(repaired_result, dict):
                repaired_array = [{"id": k, "t": v} for k, v in repaired_result.items()]
            elif isinstance(repaired_result, list):
                repaired_array = repaired_result
            else:
                raise AgentResultError(f"Agent返回结果不是array的json形式, result: {result}")

            # 检查是否与原文完全相同（疑似翻译失败）
            # 按ID排序后比较，确保可靠性
            sorted_original = sorted([{"id": k, "t": v} for k, v in original_chunk.items()], key=lambda x: x["id"])
            sorted_result = sorted(repaired_array, key=lambda x: x.get("id", ""))
            if sorted_original == sorted_result:
                raise AgentResultError("翻译结果与原文完全相同，疑似翻译失败，将进行重试。")

            # 转换为 dict 便于处理
            result_dict = {item["id"]: item["t"] for item in repaired_array if "id" in item and "t" in item}
            original_keys = set(original_chunk.keys())
            result_keys = set(result_dict.keys())

            # 如果ID不完全匹配
            if original_keys != result_keys:
                # 构造一个最完整的"部分结果"
                final_chunk = {}
                common_keys = original_keys.intersection(result_keys)
                missing_keys = original_keys - result_keys
                extra_keys = result_keys - original_keys

                # 只保留原文有的ID，丢弃多余的ID（可能是模型幻觉）
                if extra_keys:
                    logger.warning(f"检测到多余的ID（可能是模型幻觉，已丢弃）: {extra_keys}")

                # 合并已翻译的部分和缺失的部分
                for key in common_keys:
                    final_chunk[key] = str(result_dict[key])
                for key in missing_keys:
                    final_chunk[key] = str(original_chunk[key])

                # 如果所有ID都匹配了，直接返回
                if not missing_keys and not extra_keys:
                    return final_chunk

                # 如果有缺失的ID，抛出部分结果异常
                if missing_keys:
                    logger.warning(f"缺失的ID: {missing_keys}")
                    raise PartialAgentResultError("ID不匹配，触发重试", partial_result=final_chunk, append_prompt=f"\nBe careful not to omit any IDs from the input; do not combine sentences when translating.\n")

            # 如果ID完全匹配（理想情况），正常返回
            for key, value in result_dict.items():
                result_dict[key] = str(value)

            return result_dict

        except (RuntimeError, JSONDecodeError) as e:
            # 对于JSON解析等硬性错误，继续抛出普通ValueError
            raise AgentResultError(f"结果处理失败: {e.__repr__()}")

    def _error_result_handler(self, origin_prompt: str, logger: Logger):
        """
        处理在所有重试后仍然失败的请求。
        作为备用方案，返回原文内容，并将所有值转换为字符串。
        """
        original_segments = get_original_segments(origin_prompt)
        if original_segments == "":
            return {}
        try:
            original_chunk = json_repair.loads(original_segments)
            # 此处逻辑保留，作为最终的兜底方案
            for key, value in original_chunk.items():
                original_chunk[key] = f"{value}"
            return original_chunk
        except (RuntimeError, JSONDecodeError):
            logger.error(f"原始prompt也不是有效的json格式: {original_segments}")
            # 如果原始prompt本身也无效，返回一个清晰的错误对象
            return {"error": f"{original_segments}"}

    def send_segments(self, segments: list[str], chunk_size: int) -> list[str]:
        indexed_originals, chunks, merged_indices_list = segments2json_chunks(segments, chunk_size)
        prompts = [generate_prompt(json.dumps(chunk, ensure_ascii=False, indent=0), self.to_lang) for chunk in chunks]
        translated_chunks = super().send_prompts(prompts=prompts, json_format=self.force_json,
                                                 pre_send_handler=self._pre_send_handler,
                                                 result_handler=self._result_handler,
                                                 error_result_handler=self._error_result_handler)

        indexed_translated = indexed_originals.copy()
        for chunk in translated_chunks:
            try:
                if not isinstance(chunk, dict):
                    self.logger.warning(f"接收到的chunk不是有效的字典，已跳过: {chunk}")
                    continue
                for key, val in chunk.items():
                    if key in indexed_translated:
                        indexed_translated[key] = val
                    else:
                        self.logger.warning(f"在结果chunk中发现未知键 '{key}'，已忽略。")
            except (AttributeError, TypeError) as e:
                self.logger.error(f"处理chunk时发生类型或属性错误，已跳过。Chunk: {chunk}, 错误: {e.__repr__()}")
            except Exception as e:
                self.logger.error(f"处理chunk时发生未知错误: {e.__repr__()}")

        # 重建最终列表
        result = []
        last_end = 0
        ls = list(indexed_translated.values())
        for start, end in merged_indices_list:
            result.extend(ls[last_end:start])
            merged_item = "".join(map(str, ls[start:end]))
            result.append(merged_item)
            last_end = end

        result.extend(ls[last_end:])
        return result

    async def send_segments_async(self, segments: list[str], chunk_size: int) -> list[str]:
        indexed_originals, chunks, merged_indices_list = await asyncio.to_thread(segments2json_chunks, segments,
                                                                                 chunk_size)
        prompts = [generate_prompt(json.dumps(chunk, ensure_ascii=False, indent=0), self.to_lang) for chunk in chunks]

        translated_chunks = await super().send_prompts_async(prompts=prompts, force_json=self.force_json,
                                                             pre_send_handler=self._pre_send_handler,
                                                             result_handler=self._result_handler,
                                                             error_result_handler=self._error_result_handler)
        indexed_translated = indexed_originals.copy()
        for chunk in translated_chunks:
            try:
                if not isinstance(chunk, dict):
                    self.logger.error(f"接收到的chunk不是有效的字典，已跳过: {chunk}")
                    continue
                for key, val in chunk.items():
                    if key in indexed_translated:
                        # 此处不再需要 str(val)，因为 _result_handler 已经处理好了
                        indexed_translated[key] = val
                    else:
                        self.logger.warning(f"在结果chunk中发现未知键 '{key}'，已忽略。")
            except (AttributeError, TypeError) as e:
                self.logger.error(f"处理chunk时发生类型或属性错误，已跳过。Chunk: {chunk}, 错误: {e.__repr__()}")
            except Exception as e:
                self.logger.error(f"处理chunk时发生未知错误: {e.__repr__()}")

        # 重建最终列表
        result = []
        last_end = 0
        ls = list(indexed_translated.values())
        for start, end in merged_indices_list:
            result.extend(ls[last_end:start])
            merged_item = "".join(map(str, ls[start:end]))
            result.append(merged_item)
            last_end = end

        result.extend(ls[last_end:])
        return result

    def update_glossary_dict(self, update_dict: dict | None):
        if self.glossary_dict is None:
            self.glossary_dict = {}
        if update_dict is not None:
            self.glossary_dict = self.glossary_dict | update_dict



# SPDX-FileCopyrightText: 2025 QinHan
# SPDX-License-Identifier: MPL-2.0
"""进度跟踪模块"""
import threading
from dataclasses import dataclass, field
from logging import Logger
from typing import Callable, Optional


@dataclass
class ProgressStep:
    """进度步骤"""
    name: str
    weight: int = 1  # 权重，默认1


class ProgressTracker:
    """
    进度跟踪器

    支持:
    - 定义多个步骤及其权重
    - 实时更新进度
    - 集成logger记录日志
    - 线程安全
    """

    def __init__(
        self,
        logger: Optional[Logger] = None,
        total_steps: int = 100,
        callback: Optional[Callable[[int, str], None]] = None
    ):
        """
        初始化进度跟踪器

        Args:
            logger: 日志记录器，用于记录进度日志
            total_steps: 总步数 (默认100，对应百分比)
            callback: 进度更新回调函数，签名为 (percent: int, message: str) -> None
        """
        self.logger = logger
        self.total_steps = total_steps
        self.callback = callback
        self._current_step = 0
        self._current_percent = 0
        self._message = ""
        self._lock = threading.Lock()
        self._steps: list[ProgressStep] = []
        self._step_weights: list[int] = []
        self._total_weight = 0
        self._completed_weight = 0

    def set_steps(self, steps: list[str | ProgressStep]) -> "ProgressTracker":
        """
        设置进度步骤

        Args:
            steps: 步骤名称列表，或 ProgressStep 列表

        Returns:
            self，便于链式调用
        """
        self._steps = [
            s if isinstance(s, ProgressStep) else ProgressStep(name=s)
            for s in steps
        ]
        self._step_weights = [s.weight for s in self._steps]
        self._total_weight = sum(self._step_weights)
        return self

    def update(
        self,
        step: Optional[int] = None,
        message: str = "",
        percent: Optional[int] = None
    ) -> "ProgressTracker":
        """
        更新进度

        Args:
            step: 步骤索引 (0-based)，设置后自动计算百分比
            message: 进度消息
            percent: 直接指定百分比 (0-100)，优先级高于 step

        Returns:
            self，便于链式调用
        """
        with self._lock:
            if percent is not None:
                self._current_percent = max(0, min(100, percent))
            elif step is not None:
                self._current_step = step
                if self._total_weight > 0:
                    # 根据权重计算百分比
                    self._completed_weight = sum(self._step_weights[:step + 1])
                    self._current_percent = int((self._completed_weight / self._total_weight) * 100)
                else:
                    self._current_percent = int(((step + 1) / len(self._steps)) * 100) if self._steps else 0
            else:
                # 递增一步
                self._current_step += 1
                if self._steps and self._current_step < len(self._steps):
                    self._current_percent = int(((self._current_step) / len(self._steps)) * 100)

            self._message = message

            # 记录日志
            if self.logger and message:
                self.logger.info(f"[{self._current_percent}%] {message}")

            # 调用回调
            if self.callback:
                self.callback(self._current_percent, message)

        return self

    @property
    def percent(self) -> int:
        """获取当前进度百分比"""
        return self._current_percent

    @property
    def message(self) -> str:
        """获取当前进度消息"""
        return self._message

    @property
    def current_step(self) -> int:
        """获取当前步骤索引"""
        return self._current_step

    def get_status(self) -> dict:
        """获取进度状态"""
        return {
            "percent": self._current_percent,
            "message": self._message,
            "current_step": self._current_step,
            "total_steps": len(self._steps),
        }

    def reset(self) -> "ProgressTracker":
        """重置进度"""
        with self._lock:
            self._current_step = 0
            self._current_percent = 0
            self._message = ""
            self._completed_weight = 0
        return self


class NullProgressTracker(ProgressTracker):
    """空进度跟踪器，用于不需要进度跟踪的场景"""

    def __init__(self):
        super().__init__(logger=None)

    def update(self, step=None, message="", percent=None):
        return self  # 不做任何操作


# 全局空进度跟踪器
null_progress = NullProgressTracker()



default_params = {
    "thinking": "disable",
    "chunk_size": 4000,
    "concurrent": 30,
    "temperature": 0.7,
    "timeout": 1200,
    "retry": 2,
    "system_proxy_enable": False,
    "extra_body": ""
}
